package com.chinacreator.video.redis;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Repository;

import java.util.concurrent.TimeUnit;

@Repository
public class RedisService {

    @Autowired
    @Qualifier("redisTemplate")
    private RedisTemplate redisTemplate;

    @Autowired
    @Qualifier("stringRedisTemplate")
    private StringRedisTemplate stringRedisTemplate;

    public void add(String key, String value) {
        stringRedisTemplate.opsForValue().set(key, value, 100, TimeUnit.SECONDS);
    }

    public String get(String key) {
        String source = stringRedisTemplate.opsForValue().get(key);
        if (!StringUtils.isEmpty(source)) {
            return source;
        }
        return null;
    }

    public long incr(String key) {
        long count = stringRedisTemplate.opsForValue().increment(key, 1);
        if (count == 1) {
            stringRedisTemplate.expire(key, 60, TimeUnit.SECONDS);
        }

        long expire = stringRedisTemplate.getExpire(key);
        if(expire <= 0){//有可能KEY没有设置过期时间，移除些KEY
            stringRedisTemplate.delete(key);
            count = stringRedisTemplate.opsForValue().increment(key, 1);
            if (count == 1) {
                stringRedisTemplate.expire(key, 60, TimeUnit.SECONDS);
            }
        }
        return count;
    }

    public long incrAdd(String key, long kl) {
        long count = stringRedisTemplate.opsForValue().increment(key, 1);
        stringRedisTemplate.expire(key, kl, TimeUnit.SECONDS);
        return count;
    }

    public long incrBack(String key, long kl) {
        long count = stringRedisTemplate.opsForValue().increment(key, -1);
        stringRedisTemplate.expire(key, kl, TimeUnit.SECONDS);
        return count;
    }

    public void removeKey(String key) {
        stringRedisTemplate.opsForValue().getOperations().delete(key);
    }

    /**
     * 哈希获取数据
     *
     * @param key
     * @param hashKey
     * @return
     */
    public Object hmGet(String key, Object hashKey) {
        HashOperations<String, Object, Object> hash = redisTemplate.opsForHash();
        return hash.get(key, hashKey);
    }

    /**
     * 判断缓存中是否有对应的value
     *
     * @param key
     * @return
     */
    public boolean exists(final String key) {
        return redisTemplate.hasKey(key);
    }

    /**
     * hash递增 如果不存在,就会创建一个 并把新增后的值返回
     *
     * @param key  键
     * @param item 项
     * @param by   要增加几(大于0)
     * @return
     */
    public double hincr(String key, String item, double by) {
        return redisTemplate.opsForHash().increment(key, item, by);
    }

    /**
     * 指定缓存失效时间
     *
     * @param key  键
     * @param time 时间(秒)
     * @return
     */
    public boolean expire(String key, long time) {
        try {
            if (time > 0) {
                redisTemplate.expire(key, time, TimeUnit.SECONDS);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * 递增
     *
     * @param key   键
     * @param delta 要增加几(大于0)
     * @return
     */
    public long incr(String key, long delta) {
        if (delta < 0) {
            throw new RuntimeException("递增因子必须大于0");
        }
        return redisTemplate.opsForValue().increment(key, delta);
    }
}

