package com.chinacreator.video.service;


import com.alibaba.fastjson.JSON;
import com.chinacreator.video.bean.ActivityLimitConfig;
import com.chinacreator.video.bean.ActivityVipConfigBean;
import com.chinacreator.video.bean.UserPrizeBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActivePptvDao;
import com.chinacreator.video.mapper.RaffleActivityConfigDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.service.handle.GetVipService;
import com.chinacreator.video.util.IpUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.List;

@Component
public class ActivityPptvService {

    private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");

    @Autowired
    private RedisService redisService;

    @Autowired
    private ComonService comonService;

    @Autowired
    private MobService mobService;

    @Autowired
    private GetVipService getVipService;
    // 放大倍数
    private static final int mulriple = 10000;


    @Autowired
    private RaffleActivityConfigDao configDao;

    @Autowired
    private ActivePptvDao activeDao;

    /**
     * 取值
     * @param key
     * @return
     * @throws BussinessException
     */
    public long getRedis(String key) throws BussinessException {
        try {
            String count = redisService.get(key);
            System.out.println(key+":"+count);
            if(StringUtils.isEmpty(count)){
                return 0;
            }
            return Long.parseLong(count);

        } catch (Exception e) {

            ErrorLOGGER.error("", e);

        }
        return 0;
    }


    /**
     * 设值
     * @param key
     * @param value
     * @throws BussinessException
     */
    public void setRedis(String key,String value) throws BussinessException {
        try {
            redisService.add(key,value);
            System.out.println(key+":"+value);
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
    }

    /**
     *  加1
     * @param key
     * @return
     * @throws BussinessException
     */
    public long addRedis(String key) throws BussinessException {
        try {
            long count = redisService.incrAdd(key,60L*60L*24L*100L);
            System.out.println(key+":"+count);
            return count;
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
        return 0;
    }

    /**
     * 减1
     * @param key
     * @return
     * @throws BussinessException
     */
    public long backRedis(String key) throws BussinessException{
        try {
            long count = redisService.incrBack(key,60L*60L*24L*100L);
            System.out.println(key+":"+count);
            return count;
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
        return 0;

    }




    /**
     * 查询检验
     * @param vipInfo
     * @param config
     * @throws BussinessException
     */
    public void checkVipQuery(VipInfo vipInfo, ActivityLimitConfig config) throws BussinessException {
        //判断活动是否在有效期内
        this.checkPptv(vipInfo,config);

        comonService.checkTimestamp(vipInfo.getTimestamp());

        comonService.checkQueryVipSign(vipInfo);
        mobService.checkVipMob(vipInfo);
    }
    public void checkVipQuery(VipInfo vipInfo) throws BussinessException {
        this.comonService.checkTimestamp(vipInfo.getTimestamp());
        this.comonService.checkQueryVipSign(vipInfo);
        this.mobService.checkVipMob(vipInfo);
    }
    public void checkVipQuery(UserPrizeBean vipInfo) throws BussinessException {
        this.checkRedis(vipInfo);
        this.comonService.checkTimestamp(vipInfo.getTimestamp());
        this.comonService.checkQueryVipSign(vipInfo);
        this.mobService.checkVipMob(vipInfo);
    }
    /**
     * 抽奖校验
     *
     * @param vipInfo
     * @throws BussinessException
     */
    public void checkVip(VipInfo vipInfo,ActivityLimitConfig config) throws BussinessException {
        this.checkRedis(vipInfo);
        this.checkPptv(vipInfo, config);
        comonService.checkTimestamp(vipInfo.getTimestamp());
        comonService.checkQueryVipSign(vipInfo);
        mobService.checkVipMob(vipInfo);
    }


    /**
     * 会员领取
     *
     * @param activityInfo
     * @throws BussinessException
     */
    public String giveVip(VipInfo activityInfo,ActivityLimitConfig config) throws BussinessException {
        String result="";
        this.queryVip(activityInfo);
        //判断cpid spid是否可以领取pptv会员
        ActivityVipConfigBean configBean=activeDao.getActiveConfig(activityInfo.getCpid(),activityInfo.getSpid());

        if(configBean!=null && configBean.getVipid()!=null && configBean.getVippwd()!=null){

           this.joinPptv(activityInfo, configBean);


        }else{
            throw new BussinessException("9088", "该cp暂未开放");
        }
        System.out.println("JoinInfo="+ JSON.toJSONString(activityInfo));
        return result;
    }


    /**
     *  抽奖
     * @param activityInfo
     * @param config
     * @return
     */
    public void joinPptv(VipInfo activityInfo,ActivityVipConfigBean config)throws BussinessException{

        //是否已经领取
        String joincount = activeDao.getJoinActive(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if(!StringUtils.isEmpty(joincount) && joincount.equals("0")) {
            throw new BussinessException("6081", "本月已领取pptv会员");
        }

        String errorinfo="";
        String resultcode="";
        String province = activeDao.getOrderPrivince(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
        try{
            getVipService.give(activityInfo,config.getVipid(),config.getVippwd());
            resultcode="0";
            errorinfo="成功";
        } catch (Exception e) {
            if(e instanceof BussinessException) {
                resultcode=((BussinessException) e).getCode();
                errorinfo=(((BussinessException) e).getMessage());
                throw e;
            }else {
                resultcode="8000";
                errorinfo=e.getMessage();
                throw e;
            }
        } finally {
            activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,resultcode,  errorinfo);
        }

    }




    /**
     * 检查是否是爱奇艺抽奖活动且有效
     * @param vipInfo
     * @throws BussinessException
     */
    public void checkPptv(VipInfo vipInfo,ActivityLimitConfig config) throws BussinessException {
        String activetype=config.getActivetype();
        String activeend=config.getActiveendtime();

        if(!activetype.equals(vipInfo.getActivetype())){
            throw new BussinessException("9083", "该活动无效");
        }

        try {
            if(new SimpleDateFormat("yyyyMMddHHmmss").parse(activeend).getTime()<System.currentTimeMillis()){
                throw new BussinessException("9083", "该活动已经结束");
            }
        } catch (Exception e) {
            if (e instanceof BussinessException) {
                throw (BussinessException)e;
            }else{
                ErrorLOGGER.error("", e);
            }
        }
    }


    /**
     *
     * @param activityInfo
     * @throws BussinessException
     */
    public void queryVip(VipInfo activityInfo) throws BussinessException {
        //是否已经领取
        String joincount = activeDao.getJoinActive(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if(!StringUtils.isEmpty(joincount) && joincount.equals("0")) {
            throw new BussinessException("6081", "本月已领取该产品pptv会员");
        }

        //是否有效订购
        int ordercount = activeDao.getOrderRelation(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
        if(ordercount<=0){
            throw new BussinessException("6082", "用户不具备领取资格");
        }



    }

    public String queryVipRecord(VipInfo activityInfo) throws BussinessException {
        List<VipInfo> list = this.activeDao.getJoinRecord(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if (list == null || list.size() == 0) {
            throw new BussinessException("6083", "您未领取此产品的pp会员噢！");
        } else {
            return JSON.toJSONString(list);
        }
    }





    public void checkRedis(VipInfo vipInfo) throws BussinessException {
        try {
            long count = redisService.incr("joinPPtvVIp_" + vipInfo.getCpid()+vipInfo.getSpid()+vipInfo.getActivetype() + "_" + vipInfo.getUserid());
            if (count > 1) {
                throw new BussinessException("8001", "并发请求，请稍后再试");
            }
        } catch (Exception e) {
            if (e instanceof BussinessException) {
                throw e;
            }else{
                ErrorLOGGER.error("", e);
            }
        }
    }



    /**
     * ip校验
     */
    public void checkIp(ActivityLimitConfig config)throws BussinessException {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        if(org.apache.commons.lang.StringUtils.isNotEmpty(config.getLimitip())) {
            List<String> list = Arrays.asList(config.getLimitip().split(","));
            System.out.println("listIP:"+list);
            String ip = IpUtil.getClinetIpByReq(request);
            System.out.println("realip:"+ip);
            if(!list.contains(ip)) {
                throw new BussinessException("10009", "ip地址校验失败"+ip);
            }
        }
    }

}
