package com.chinacreator.video.service;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.OrderInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.OrderDao2;
import com.chinacreator.video.util.AesUtilByIntensive;
import com.chinacreator.video.util.HttpInvoke;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

@Service
public class JiyueService {

    private static Logger log = Logger.getLogger("jiyueOrder");

    @Autowired
    private OrderDao2 orderDao;

    public void order(OrderInfo orderInfo) throws Exception{
        Map<String, String> resMap = new HashMap<String, String>();
        Map logMap = new HashMap(); //日志记录
        try {
            orderInfo.setInvokeface("order");
            logMap.put("userid",orderInfo.getUserid());
            logMap.put("spid",orderInfo.getSpid());
            logMap.put("cpid",orderInfo.getCpid());
            //获取接口配置信息
            Map invokeUrlConf = this.getInvokeConf(orderInfo);
            logMap.put("invokeurl", invokeUrlConf.get("INVOKEURL"));
            JSONObject paramsJson = new JSONObject();
            JSONObject dataJson = this.buildOrderParam(orderInfo, invokeUrlConf);
            logMap.put("serialid",dataJson.getString("serialid"));
            logMap.put("data", dataJson.toJSONString());
            String param1 = invokeUrlConf.get("PARAM1").toString();
            String cpid = JSON.parseObject(param1).getString("cpid"); //合作方编码，字符串类型，不超过16个字符（由联通分配）
            String aespwd = JSON.parseObject(param1).getString("aespwd"); //AES加密KEY
            paramsJson.put("cpid", cpid);
            paramsJson.put("data", AesUtilByIntensive.encrypt(dataJson.toJSONString(), aespwd));
            String params = paramsJson.toJSONString();
            logMap.put("reqParams", params);
            //调接口
            String rspParams = this.invoke(params, invokeUrlConf, orderInfo);
            logMap.put("rspParams", rspParams);

            if(!StringUtils.isEmpty(rspParams)){
                JSONObject resObject = JSON.parseObject(rspParams);
                resMap.put("resultCode",resObject.getString("resultCode"));
                resMap.put("errorInfo",resObject.getString("errorInfo"));

                if(resObject.getString("data") != null){
                    String data = resObject.getString("data");
                    //解密返回参数
                    String deoutparams = AesUtilByIntensive.decrypt(data, aespwd);
                    //设置到日志类
                    JSONObject dataObject = JSONObject.parseObject(deoutparams);
                    resMap.put("data", dataObject.toJSONString());
                }
                if (!"0000".equals(resMap.get("resultCode"))&&!(rspParams.contains("不可以重复订购"))){
                    throw new BussinessException(resMap.get("resultCode"), resMap.get("errorInfo"));
                }else {
                    throw new BussinessException("0", "成功");
                }
            }else {
                throw new BussinessException("9170", "调用前置订购接口异常");
            }
        } finally {
            log.info(JSON.toJSONString(logMap));
        }

    }

    public void cancel(OrderInfo orderInfo) throws Exception{
        Map<String, String> resMap = new HashMap<String, String>();
        Map logMap = new HashMap(); //日志记录
        try {
            orderInfo.setInvokeface("cancel");
            //获取接口配置信息
            Map invokeUrlConf = this.getInvokeConf(orderInfo);
            logMap.put("invokeurl", invokeUrlConf.get("INVOKEURL"));
            JSONObject paramsJson = new JSONObject();
            JSONObject dataJson = this.buildCancelParams(orderInfo, invokeUrlConf);
            logMap.put("data", dataJson.toJSONString());
            String param1 = invokeUrlConf.get("PARAM1").toString();
            String cpid = JSON.parseObject(param1).getString("cpid"); //合作方编码，字符串类型，不超过16个字符（由联通分配）
            String aespwd = JSON.parseObject(param1).getString("aespwd"); //AES加密KEY
            paramsJson.put("cpid", cpid);
            paramsJson.put("data", AesUtilByIntensive.encrypt(dataJson.toJSONString(), aespwd));
            String params = paramsJson.toJSONString();
            logMap.put("reqParams", params);
            //调接口
            String rspParams = this.invoke(params, invokeUrlConf, orderInfo);
            logMap.put("rspParams", rspParams);
            if(!StringUtils.isEmpty(rspParams)){
                JSONObject resObject = JSON.parseObject(rspParams);
                resMap.put("resultCode",resObject.getString("resultCode"));
                resMap.put("errorInfo",resObject.getString("errorInfo"));

                if(resObject.getString("data") != null){
                    String data = resObject.getString("data");
                    //解密返回参数
                    String deoutparams = AesUtilByIntensive.decrypt(data, aespwd);
                    //设置到日志类
                    JSONObject dataObject = JSONObject.parseObject(deoutparams);
                    resMap.put("data", dataObject.toJSONString());
                }
                if (!"0000".equals(resMap.get("resultCode"))){
                    throw new BussinessException(resMap.get("resultCode"), resMap.get("errorInfo"));
                }else {
                    throw new BussinessException("0", "成功");
                }

            }else {
                throw new BussinessException("9170", "调用前置退订接口异常");
            }
        }finally {
            log.info(JSON.toJSONString(logMap));
        }
    }


    private String invoke(String params, Map invokeUrlConf, OrderInfo orderInfo) throws Exception {
        Map reqProperty = this.getProperty();
        String rspParams = ""; //响应报文
        String reqParams = params; //请求报文
        try {
            reqParams = params;
            String serviceUrl = invokeUrlConf.get("INVOKEURL").toString();
            rspParams = HttpInvoke.sendHttpByPost("POST", serviceUrl, reqParams, reqProperty);
        } catch (Exception e) {
            e.printStackTrace();
            log.error(params+",调集约平台接口出现异常，" + e);
            throw new BussinessException("8000", "调集约平台接口出现异常，" + e.getMessage());
        }
        return rspParams;
    }

    private Map getProperty() {
        Map reqProperty = new HashMap();
        reqProperty.put("Content-Type", "application/json; charset=UTF-8");
        reqProperty.put("Accept", "application/json");
        return reqProperty;
    }

    private JSONObject buildOrderParam(OrderInfo orderInfo, Map invokeUrlConf) throws Exception {
        String param1 = invokeUrlConf.get("PARAM1").toString();
        String cpid = JSON.parseObject(param1).getString("cpid"); //合作方编码，字符串类型，不超过16个字符（由联通分配）
        String aespwd = JSON.parseObject(param1).getString("aespwd"); //AES加密KEY

        HashMap product = orderDao.getProduct(orderInfo);
        if (product!=null&&product.size()>0) {
            orderInfo.setVacid(product.get("PRODUCTID").toString());
        }else {
            orderInfo.setVacid(orderInfo.getSpid());
        }
        String busiid = JSON.parseObject(param1).getString("busiid"); //	String(20)	是	接口对业务操作的标记，用于区分调用的接口，取值在备注中
        String serialid = orderDao.getserialid(); //	String（50）	是	本次请求的流水号，由业务单元生成，需要唯一
        String usermob = orderInfo.getUserid(); //	String(12)	是	用户手机号码
        String productid = orderInfo.getVacid(); //	String(20)	是	产品编码（不同产品对应不同的编码，联通分配）
        String festivalid = ""; //	String(50)	否	活动编码
        String channel = JSON.parseObject(param1).getString("channel"); //String(4)	是	推广渠道，允许为空
        String token = orderInfo.getToken(); //	String(20)	是	CAP二次认证生成的token
        String righttypeid = ""; //	String(50)	否	权益类型编码,如腾讯会员
        String rightaccount = ""; //	String(50)	否	领取账号
        String staffid = JSON.parseObject(param1).getString("staffid"); //	String(20)	否	CB可识别的工号，附加产品必传，增值产品可选
        String sourcedeviceid = JSON.parseObject(param1).getString("sourcedeviceid"); //	String(6)	是	CAP经营单元设备类型，调用CAP订购接口参数，由CAP分配
        String systemid	= JSON.parseObject(param1).getString("systemid"); //String(10)	是	系统编码，调用CAP订购接口参数，由CAP分配
        String accesstype = orderInfo.getAccesstype(); //	String(1)	是	CAP二次认证方式
        if (StringUtils.isEmpty(accesstype)){
            accesstype = "1";
        }
        /***
         1：H5认证
         2：SDK认证
         3：短信认证
         4：页面认证（与CONVEYSTRING字段关联）
         5：频率页面认证（小额产品）
         6：免二次认证（2B产品）
         ***/
        String ordermethod = orderInfo.getOrderchannel(); //	String(2)	是	订购渠道:
        /***
         01 WEB
         02 短信
         03 WAP
         04 IVR
         05 APP
         ***/
        String jiyueChannel = orderDao.getChannel(orderInfo.getSpid());
        if (!StringUtils.isEmpty(jiyueChannel)){
            channel = jiyueChannel;
        }
        if ("boyuan".equals(cpid)){
            channel = orderInfo.getSubchannel();
            if (StringUtils.isEmpty(channel)) {
                channel = orderInfo.getOrderchannel();
            }
        }
        if (StringUtils.isEmpty(channel)&&StringUtils.isEmpty(orderInfo.getChannel())){
            channel = "boyuan";
        }
        System.out.println("========================="+jiyueChannel);
        JSONObject data = new JSONObject();
        data.put("busiid", busiid);
        data.put("serialid", serialid);
        data.put("usermob", usermob);
        data.put("productid", productid);
        data.put("festivalid", festivalid);
        data.put("channel", channel);
        data.put("token", token);
        data.put("righttypeid", righttypeid);
        data.put("rightaccount", rightaccount);
        data.put("staffid", staffid);
        data.put("sourcedeviceid", sourcedeviceid);
        data.put("systemid", systemid);
        data.put("accesstype", accesstype);
        data.put("ordermethod", ordermethod);
        String touTiaoSpid = orderDao.getDicValue("touTiaoSpid");
        String[] split = touTiaoSpid.split(",");
        for (String str:split) {
            String[] strings = str.split("_");
            if (strings[0].equals(orderInfo.getSpid())){
                String staffid2 = JSON.parseObject(param1).getString("staffid2");
                data.put("staffid", staffid2);
                break;
            }
        }

        return data;
    }

    /**
     * 组建退订调接口参数，返回参数JSON对象
     * @param orderInfo
     * @return
     */
    private JSONObject buildCancelParams(OrderInfo orderInfo, Map invokeUrlConf) throws Exception {
        String param1 = invokeUrlConf.get("PARAM1").toString();
        String cpid = JSON.parseObject(param1).getString("cpid"); //合作方编码，字符串类型，不超过16个字符（由联通分配）
        String aespwd = JSON.parseObject(param1).getString("aespwd"); //AES加密KEY


        String busiid = JSON.parseObject(param1).getString("busiid"); //	String(20)	是	接口对业务操作的标记，用于区分调用的接口，取值在备注中
        String serialid = orderDao.getserialid(); //	String（50）	是	本次请求的流水号，由业务单元生成，需要唯一
        String usermob = orderInfo.getUserid(); //	String(12)	是	用户手机号码
        HashMap product = orderDao.getProduct(orderInfo);
        if (product!=null&&product.size()>0) {
            orderInfo.setVacid(product.get("PRODUCTID").toString());
        }else {
            orderInfo.setVacid(orderInfo.getSpid());
        }
        String productid = orderInfo.getVacid(); //	String(20)	是	产品编码（不同产品对应不同的编码，联通分配）

        String channel = JSON.parseObject(param1).getString("channel"); //String(4)	是	退订渠道
        if(StringUtils.isEmpty(channel)){
            channel = "";
        }
        String jiyueChannel = orderDao.getChannel(orderInfo.getSpid());
        if (!StringUtils.isEmpty(jiyueChannel)){
            channel = jiyueChannel;
        }
        if ("boyuan".equals(cpid)){
            channel = orderInfo.getSubchannel();
            if (StringUtils.isEmpty(channel)){
                channel = orderInfo.getCancelchannel();
            }
        }
        if (StringUtils.isEmpty(channel)&&StringUtils.isEmpty(orderInfo.getChannel())){
            channel = "boyuan";
        }
        System.out.println("========================="+jiyueChannel);
        String staffid = JSON.parseObject(param1).getString("staffid"); //	String(20)	否	CB可识别的工号，附加产品必传，增值产品可选
        String sourcedeviceid = JSON.parseObject(param1).getString("sourcedeviceid"); //	String(6)	是	CAP经营单元设备类型，调用CAP订购接口参数，由CAP分配
        String systemid	= JSON.parseObject(param1).getString("systemid"); //String(10)	是	系统编码，调用CAP订购接口参数，由CAP分配

        JSONObject data = new JSONObject();
        data.put("busiid", busiid);
        data.put("serialid", serialid);
        data.put("usermob", usermob);
        data.put("productid", productid);
        data.put("channel", channel);
        data.put("staffid", staffid);
        data.put("sourcedeviceid", sourcedeviceid);
        data.put("systemid", systemid);

        String touTiaoSpid = orderDao.getDicValue("touTiaoSpid");
        String[] split = touTiaoSpid.split(",");
        for (String str:split) {
            String[] strings = str.split("_");
            if (strings[0].equals(orderInfo.getSpid())){
                String staffid2 = JSON.parseObject(param1).getString("staffid2");
                data.put("staffid", staffid2);
                break;
            }
        }

        return data;
    }
    /**
     * 获取接口配置信息
     * @param orderInfo
     * @return
     * @throws Exception
     */
    private Map getInvokeConf(OrderInfo orderInfo)throws Exception {
        Map invokeUrlConf = orderDao.getInvokeUrl(orderInfo.getInvokeface(), orderInfo.getSpid());
        if(invokeUrlConf == null || invokeUrlConf.size() == 0){
            invokeUrlConf = orderDao.getInvokeUrl(orderInfo.getInvokeface(), "0");
        }
        return invokeUrlConf;
    }
}
