package com.chinacreator.video.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.ActivityBean;
import com.chinacreator.video.bean.CpSpBean;
import com.chinacreator.video.bean.OrderInfo;
import com.chinacreator.video.bean.WhiteBean;
import com.chinacreator.video.dto.QueryDataOutPipe;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.log.OrderLog;
import com.chinacreator.video.mapper.*;
import com.chinacreator.video.util.URLUtil;
import com.chinacreator.video.util.VacUtil;
import com.chinacreator.video.validate.ChannelOrderValidate;
import com.chinacreator.video.validate.ProvinceValidate;

import java.util.*;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.time.DateFormatUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class OrderService {

	private static Logger log = Logger.getLogger(OrderService.class);

	@Autowired
	private ComonService comonService;

	@Autowired
	private CpSpService cpSpService;

	@Autowired
	private MobService mobService;

	@Autowired
	private OrderDao orderDao;

	@Autowired
	private PreOrderDao preOrderDao;

	@Autowired
	private WhiteDao whiteDao;

	@Autowired
	private CpSpDao cpSpDao;

	@Autowired
	private SysDao sysDao;

	@Autowired
	private AreaDao areaDao;

	@Autowired
	private QueryOrderDao queryOrderDao;

	@Autowired
	private ActiveDao activeDao;

	@Autowired
	private OrderLogDao orderLogDao;

	@Autowired
	private QueryPreOrderDao queryPreOrderDao;

	@Autowired
	private EcbService ecbService;

	@Autowired
	private VacControlService vacControlService;

	@Autowired
	private CctvActivityService cctvActivityService;

	@Autowired
	private EcbBalanceService ecbBalanceService;

	@Autowired
	private ProvinceValidate provinceValidate;

	@Autowired
	private ChannelOrderValidate channelOrderValidate;

	@Autowired
	private UnifiedAuthService unifiedAuthService;

	@Autowired
	private CapOrderService capOrderService;

	@Autowired
	private CapKbxkQryService capKbxkQryService;

	@Autowired
	private DictionaryDao dictionaryDao;

	@Autowired
	private CommConfDao commConfDao;
	
	public void orderCheck(OrderInfo orderInfo) throws BussinessException {
		comonService.checkOrder(orderInfo);
		cpSpService.checkSp(orderInfo);
		mobService.checkMob(orderInfo);
		Map<String, String> map = areaDao.getArea(orderInfo.getUserid());
		if (map != null) {
			orderInfo.setProvince(map.get("province") == null ? "" : map.get("province"));
			orderInfo.setArea(map.get("area") == null ? "" : map.get("area"));
		}
		if (orderInfo.getCpid().equals("twowei")) {
			if (orderInfo.getCanceltype() == 1) {
				orderInfo.setCpid("tencent");
				orderInfo.setSpid("6");
			} else {
				orderInfo.setCpid("tencent");
				orderInfo.setSpid("101");
			}
		}
		//省份限制验证
		comonService.valiProvinceLimit(orderInfo);
	}
 
	/**
	 * 订购
	 * @param orderInfo
	 * @throws Exception
	 */
	public void order(OrderInfo orderInfo) throws Exception {
		String webchannel = orderInfo.getChannel();
		String resultcode = "0";
		String errrorinfo = "";
		try {
			provinceValidate.provinceValidate(orderInfo.getCpid(), orderInfo.getSpid(), orderInfo.getProvince());
			channelOrderValidate.channelValidate(orderInfo.getCpid(), orderInfo.getSpid(), orderInfo.getChannel(),
					orderInfo.getProvince());
			List<OrderInfo> localOrderInfos = orderDao.getOrderByUserid(orderInfo.getUserid());
			CpSpBean cpspBean = cpSpDao.getCpSp(orderInfo.getCpid(), orderInfo.getSpid());
			OrderInfo oldorderInfo = null;
			if (localOrderInfos != null && localOrderInfos.size() > 0) {
				for (OrderInfo localOrderInfo : localOrderInfos) {
					if (localOrderInfo.getSpid().equals(orderInfo.getSpid()) && localOrderInfo.getStatus() == 0) {
						throw new BussinessException("9060", "用户已订购");
					} else if (localOrderInfo.getSpid().equals(orderInfo.getSpid()) && localOrderInfo.getStatus() != 0
							&& NumberUtils.toLong(localOrderInfo.getCanceltime().substring(0, 6)) == NumberUtils
							.toLong(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss").substring(0, 6))) {
						oldorderInfo = localOrderInfo;
						if (orderInfo.getOrdertype() == 2 && vacControlService.isVac(orderInfo)) {
							oldorderInfo.setStatus(2);
						}
						// if (localOrderInfo.getCancelchannel() != null
						// && (localOrderInfo.getCancelchannel().equals("lvac")
						// || localOrderInfo.getCancelchannel().equals("ecb"))) {
						// oldorderInfo.setStatus(2);
						// }
					} else if (localOrderInfo.getSpid().equals(orderInfo.getSpid()) && localOrderInfo.getStatus() == 1
							&& NumberUtils.toLong(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss")) < NumberUtils
							.toLong(localOrderInfo.getCanceltime())) {
						throw new BussinessException("9065", "订购时间早于退订时间");
					} else {
						String[] mutexs = orderInfo.getMutex() == null ? null : orderInfo.getMutex().trim().split(",");
						if (mutexs != null && mutexs.length > 0
								&& Arrays.asList(mutexs).contains(localOrderInfo.getSpid())) {
							throw new BussinessException("9061", "用户已订购相同产品或者未失效");
						}
					}
				}
			}
			System.out.println("1");
			WhiteBean whiteBean = whiteDao.getUser(orderInfo.getUserid(), orderInfo.getCpid());
			if (whiteBean != null && whiteBean.getType().equals("1")) {
				throw new BussinessException("9065", "黑名单用户禁止订购");				
			}
			System.out.println("2");
			if ((orderInfo.getOrdertype()==1 ||(orderInfo.getOrdertype()==2 && ((oldorderInfo == null) || (oldorderInfo.getStatus() == 2))))
					&& ((whiteBean == null) || (whiteBean.getVac().equals("1")))) {
				String brand = ""; //品牌
				System.out.println("3");
				if(oldorderInfo == null) {
					//查询是否三户套餐限制
					brand = this.ecbService.queryEcb(orderInfo.getUserid(), orderInfo.getCpid(), orderInfo.getSpid(),
							orderInfo.getProvince());
				}
				System.out.println("4");
				//查询余额是否足够
				ecbBalanceService.queryEcbBalance(orderInfo.getUserid(), orderInfo.getCpid(), orderInfo.getSpid(),
						orderInfo.getProvince());
				System.out.println("5");
				vacControlService.canOrder(orderInfo);
				String accesstype = "1"; //sp/cp的二次认证方式: 	1：H5认证 2：SDK认证 3：短信认证  4：页面认证
				if(!StringUtils.isEmpty(orderInfo.getSeqnumber())){
					accesstype = "3";
				}
				System.out.println("6");
				//调用cap综合查询接口和卡部信控接口保存数据
				capKbxkQryService.capKbxkQry(orderInfo.getUserid(),orderInfo.getCpid(),orderInfo.getSpid());

				String result = "";
				/*boolean hasvac = true; //是否调老vac接口
				if(capOrderService.hasCap(orderInfo)){//产品配置是走CAP
					orderInfo.setChannel("CAP");
					hasvac = false;
					if(brand != null && !"".equals(brand)){
						if(brand.indexOf("4G") != -1){ //是4G用户
							result = this.capOrderService.order(orderInfo, accesstype);
							if("0008".equals(result)){ //不是4G用户，走原vac
								hasvac = true;
							}
						}else{//不是4G用户
							hasvac = true;
						}
					}else{//不能确定是否4G用户，走CAP接口
						result = this.capOrderService.order(orderInfo, accesstype);
						if("0008".equals(result)){ //不是4G用户，走原vac
							hasvac = true;
						}
					}
				}
				log.info("userid=>"+orderInfo.getUserid()+"，brand=>"+brand+"，accesstype=>"+accesstype+"，hasvac=>"+hasvac);
				if(hasvac && unifiedAuthService.sycnOrder(orderInfo)){ //没有走CAP业务配置，或者不是4G用户，只有调vac才走统一认证验证
					orderInfo.setChannel("VAC");
					result = VacUtil.vac(orderInfo.getUserid(), cpspBean.getSpcode(), cpspBean.getVacproductid(), cpspBean.getType(), orderInfo.getId() == null ? "" : orderInfo.getId());
					if(result.equals("1200")) {
						result="0";
					}
					vacControlService.saveVacOrder(orderInfo, result);
				}
*/

				//统一认证订单同步
				/*if (unifiedAuthService.sycnOrder(orderInfo)) {
					result = VacUtil.vac(orderInfo.getUserid(), cpspBean.getSpcode(), cpspBean.getVacproductid(), cpspBean.getType(), orderInfo.getId() == null ? "" : orderInfo.getId());
				}
				if(result.equals("1200")) {
					result="0";
				}
				vacControlService.saveVacOrder(orderInfo, result);
				*/

                orderInfo.setChannel("CAP");
				orderInfo.setOrderchannel(webchannel);
                //1为CAP短信接口，2为自有短信接口
                if("1".equals(orderInfo.getSubtype())){
                	 orderInfo.setChannel("SMSCAP"); //代表走的是CAP短信订购接口获取TOKEN再调CAP接口订购
                }
				System.out.println("7");
               this.capOrderService.order(orderInfo, accesstype);
/*				if (!result.equals("0")) {
					if(!"cardbus".equals(orderInfo.getOrderchannel())){
						insertPushTw(orderInfo);
					}
					if("8005".equals(result)){
						throw new BussinessException(result, "订购失败:VAC参数验证失败");
					}
					throw new BussinessException(result, "订购失败，VAC错误");
				}*/
			}

			orderInfo.setOrdertime(sysDao.getSysTime());
			orderInfo.setEffecttime(sysDao.getEfftTime());
//			orderInfo.setOrderchannel(orderInfo.getChannel());
			System.out.println("8");
			orderDao.order(orderInfo);
			if (orderInfo.getCpid().equals("tencent")
					&& (orderInfo.getSpid().equals("84") || orderInfo.getSpid().equals("101"))) {
				System.out.println("9");
				if (oldorderInfo == null) {
					activeDao.orderActivity(getActivityBean(orderInfo, 0));
				} else {
					activeDao.orderActivity(getActivityBean(orderInfo, 1));
				}
			} else if (orderInfo.getCpid().equals("cctv") && orderInfo.getSpid().equals("10081")) {
				cctvActivityService.activity(orderInfo, "0");
			}
			System.out.println("10");
		} catch (Exception e) {
			resultcode = "8000";
			errrorinfo = "系统忙,请稍后再试";
			if ((e instanceof BussinessException)) {
				resultcode = ((BussinessException) e).getCode();
				errrorinfo = ((BussinessException) e).getMessage();
				if (errrorinfo.length() > 200) {
					errrorinfo = errrorinfo.substring(0, 200);
				}
				if(errrorinfo.contains("cap订购失败")){
					String errorinfo2 = "cap订购失败";
					throw new BussinessException(resultcode,errorinfo2);
				}
			}
			throw e;
		} finally {
        	//调CAP平台或能力平台返回了指定错误信息调CAP查询接口，如果有订购关系，则把数据添加到CAP反向通知表，使用反向通知流程落订购关系，此次流程不记录订购日志，以提高订购成功率
        	if("9666".equals(resultcode) && ("CAP".equals(orderInfo.getChannel()) || "SHARE".equals(orderInfo.getChannel()))){
        		log.info("调CAP或调能力平台接口返回指定错误信息查询CAP落订购关系，不记录订购日志，userid=>"+orderInfo.getUserid()+" : "+orderInfo.getSpid());
        	}else{
    			saveLog(orderInfo, resultcode, errrorinfo, "0");
        	}
		}
	}

	public ActivityBean getActivityBean(OrderInfo orderInfo, int type) {
		ActivityBean bean = new ActivityBean();
		bean.setUserid(orderInfo.getUserid());
		bean.setCpid(orderInfo.getCpid());
		bean.setSpid(orderInfo.getSpid());
		bean.setProvince(orderInfo.getProvince());
		bean.setOrdertime(orderInfo.getOrdertime());
		bean.setArea(orderInfo.getArea());
		if (orderInfo.getCpid().equals("cctv")) {
			bean.setActiveType("8");
		} else {
			if (orderInfo.getSpid().equals("101")) {
				bean.setActiveType("11");
			} else {
				bean.setActiveType("6");
			}
		}
		bean.setOrderchannel(orderInfo.getOrderchannel());
		if (type == 0) {
			bean.setType("0");
			bean.setVipstatus("2");
		} else {
			bean.setType("1");
		}
		System.out.println(JSON.toJSONString(bean));
		return bean;
	}

	public ActivityBean getCacenlActivityBean(OrderInfo orderInfo) {
		ActivityBean bean = new ActivityBean();
		bean.setUserid(orderInfo.getUserid());
		bean.setCpid(orderInfo.getCpid());
		bean.setSpid(orderInfo.getSpid());
		if (orderInfo.getCpid().equals("cctv")) {
			bean.setActiveType("8");
		} else {
			if (orderInfo.getSpid().equals("101")) {
				bean.setActiveType("11");
			} else {
				bean.setActiveType("6");
			}
		}
		bean.setCancelchannel(orderInfo.getCancelchannel());
		bean.setCanceltime(orderInfo.getCanceltime());
		bean.setEndtime(orderInfo.getEndtime());
		return bean;
	}

	/**
	 * 预订购
	 * @param orderInfo
	 * @throws Exception
	 */
	public void preOrder(OrderInfo orderInfo) throws Exception {
			throw new BussinessException("10111", "预定功能已下线");

	}

	/**
	 * 退订
	 * @param orderInfo
	 * @throws Exception
	 */
	public void cancel(OrderInfo orderInfo) throws Exception {
		String resultcode = "0";
		String errrorinfo = "";
		String webChannel = orderInfo.getChannel();
		try {
			OrderInfo localOrderInfo = orderDao.getOrder(orderInfo);
			if (localOrderInfo == null) {
				throw new BussinessException("9063", "用户没有订购");
			}
			if (localOrderInfo.getStatus() == 1) {
				throw new BussinessException("9064", "用户已退订");
			}

			WhiteBean whiteBean = whiteDao.getUser(orderInfo.getUserid(), orderInfo.getCpid());
			if (orderInfo.getCanceltype() == 1 && (whiteBean == null || whiteBean.getVac().equals("1"))) {
				CpSpBean cpspBean = cpSpDao.getCpSp(orderInfo.getCpid(), orderInfo.getSpid());

				String result = ""; //调接口结果编码
/*				//退订，选择走CAP接口或是VAC接口
				boolean hasvac = true; //是否调老vac接口
				if(capOrderService.hasCap(orderInfo)){//产品配置是走CAP
					orderInfo.setChannel("CAP");
					hasvac = false;
					result = this.capOrderService.cancel(orderInfo);
					if("0008".equals(result)){ //不是4G用户，走原vac
						hasvac = true;
					}
				}
				if(hasvac){ //没有走CAP业务配置，或者不是4G用户
					orderInfo.setChannel("VAC");
					result = VacUtil.vac(orderInfo.getUserid(), cpspBean.getSpcode(), cpspBean.getVacproductid(),"2");
				}
				//String result = VacUtil.vac(orderInfo.getUserid(), cpspBean.getSpcode(), cpspBean.getVacproductid(),"2");
				if (!result.equals("0") && !result.equals("1201")) {
					if("8005".equals(result)){
						throw new BussinessException(result, "退订失败:VAC参数验证失败");
					}
					throw new BussinessException(result, "退订失败，VAC错误");
				}*/
              orderInfo.setChannel("CAP");
              orderInfo.setCancelchannel(webChannel);
              //1为CAP短信接口，2为自有短信接口
              if("1".equals(orderInfo.getSubtype())){
              	 orderInfo.setChannel("SMSCAP"); //代表走的是CAP短信订购接口获取TOKEN再调CAP接口订购
              }
              this.capOrderService.cancel(orderInfo);
			}

			orderInfo.setCanceltime(sysDao.getSysTime());
			orderInfo.setEndtime(sysDao.getLastDay());
//			orderInfo.setCancelchannel(orderInfo.getChannel());
			orderDao.cancel(orderInfo);
			if (orderInfo.getCpid().equals("tencent")
					&& (orderInfo.getSpid().equals("84") || orderInfo.getSpid().equals("101"))) {
				activeDao.canceActivity(getCacenlActivityBean(orderInfo));
			} else if (orderInfo.getCpid().equals("cctv") && orderInfo.getSpid().equals("10081")) {
				cctvActivityService.activity(orderInfo, "1");
			}
		} catch (Exception e) {
			resultcode = "8000";
			errrorinfo = "系统忙,请稍后再试";
			if ((e instanceof BussinessException)) {
				resultcode = ((BussinessException) e).getCode();
				errrorinfo = ((BussinessException) e).getMessage();
				if (errrorinfo.length() > 200) {
					errrorinfo = errrorinfo.substring(0, 200);
				}
				if(errrorinfo.contains("cap退订失败")){
					String errorinfo2 = "cap退订失败";
					throw new BussinessException(resultcode,errorinfo2);
				}
			}
			throw e;
		} finally {
			saveLog(orderInfo, resultcode, errrorinfo, "1");
		}
	}

	/**
	 * 预退订
	 * @param orderInfo
	 * @throws BussinessException
	 */
	public void preCancel(OrderInfo orderInfo) throws BussinessException {
		OrderInfo localOrderInfo = preOrderDao.getOrder(orderInfo.getUserid(), orderInfo.getCpid(),
				orderInfo.getSpid());
		if (localOrderInfo == null) {
			throw new BussinessException("9067", "用户没有订购");
		}
		if (localOrderInfo.getStatus() == 1) {
			throw new BussinessException("9068", "已取消预订购");
		}
		if (NumberUtils.toLong(localOrderInfo.getOrdertime()) > NumberUtils
				.toLong(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss"))) {
			throw new BussinessException("9066", "退订时间早于订购时间");
		}
		orderInfo.setCanceltime(sysDao.getSysTime());
		orderInfo.setEndtime(sysDao.getLastDay());
		orderInfo.setCancelchannel(orderInfo.getChannel());
		preOrderDao.cancel(orderInfo);
		if (orderInfo.getCanceltype() == 5) {
			if (orderInfo.getCpid().equals("tencent") && orderInfo.getSpid().equals("84")) {
				activeDao.canceActivity(getCacenlActivityBean(orderInfo));
			}
		}
	}

	public List<QueryDataOutPipe> queryOrder(OrderInfo orderInfo) {
		List<QueryDataOutPipe> list = new ArrayList<>();
		List<QueryDataOutPipe> listQueryDataOutPipe = queryOrderDao.getOrder(orderInfo);
		if (listQueryDataOutPipe != null && listQueryDataOutPipe.size() > 0) {
			for (QueryDataOutPipe queryDataOutPipe : listQueryDataOutPipe) {
				list.add(queryDataOutPipe);
			}
		}
		List<QueryDataOutPipe> prelistQueryDataOutPipe = queryPreOrderDao.getOrder(orderInfo);
		if (prelistQueryDataOutPipe != null && prelistQueryDataOutPipe.size() > 0) {
			for (QueryDataOutPipe queryDataOutPipe : prelistQueryDataOutPipe) {
				if (queryDataOutPipe.getOrderstatus().equals("0")) {
					queryDataOutPipe.setOrderstatus("2");
				} else {
					queryDataOutPipe.setOrderstatus("3");
				}
				list.add(queryDataOutPipe);
			}
		}
		return list;
	}




	/**
	 * VAC订购失败推送拓维后向订购
	 * @param orderInfo
	 */
	public void insertPushTw(OrderInfo orderInfo){

		try {

			Map<String, String> map = new HashMap<String, String>();
			map.put("userid",orderInfo.getUserid());
			map.put("kcSpid",orderInfo.getSpid());
			map.put("kcCpid",orderInfo.getCpid());
			map.put("type", "twhx");
			String  mqReciveUrl = dictionaryDao.getValue("mqReciveUrl");
			URLUtil.post(mqReciveUrl, JSON.toJSONString(map));
			//测试环境
//			URLUtil.post("http://10.199.99.158:8030/mq-service/recive.do", JSON.toJSONString(map));
			//生产环境
//			URLUtil.post("http://10.199.99.144:8090/mq-service/recive.do", JSON.toJSONString(map));

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * 写日志
	 * @param orderInfo
	 * @param resultCode
	 * @param errorInfo
	 * @param type
	 */
	public void saveLog(OrderInfo orderInfo, String resultCode, String errorInfo, String type) {
		OrderLog orderLog = new OrderLog();
		orderLog.setApptype("");
//		if(orderInfo.getOrderchannel() != null){
//			orderLog.setChannel(orderInfo.getOrderchannel());
//		}else{
//			orderLog.setChannel(orderInfo.getCancelchannel());
//		}

		if (type.equals("0")) {
			orderLog.setStatus(0);
			orderLog.setChannel(orderInfo.getOrderchannel());
			orderLog.setOrderstatus(2);
		} else {
			orderLog.setStatus(1);
			orderLog.setChannel(orderInfo.getCancelchannel());
			orderLog.setOrderstatus(4);
		}
		orderLog.setArea(orderInfo.getArea());
		orderLog.setCpid(orderInfo.getCpid());
		orderLog.setIsexperience(orderInfo.getIsexperience());
		orderLog.setOrdertype(orderInfo.getOrdertype() + "");
		orderLog.setProvince(orderInfo.getProvince());
		orderLog.setArea(orderInfo.getArea());
		orderLog.setSpid(orderInfo.getSpid());
		orderLog.setUserid(orderInfo.getUserid());
		orderLog.setErrorcode(resultCode);
		orderLog.setErrorinfo(errorInfo);
		if (StringUtils.isNotEmpty(orderInfo.getTimes())) {
			orderLog.setTimes((System.currentTimeMillis() - NumberUtils.toLong(orderInfo.getTimes())) + "");
		}
		System.out.println("orderLog=" + JSON.toJSONString(orderLog));
		orderLogDao.insertLog(orderLog);
	}
	
	public static void main(String args[]){
		System.out.println("----");
	}

}
