package com.chinacreator.video.service;


import java.net.URLEncoder;

import org.apache.commons.lang.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.chinacreator.video.bean.ActivityConfigBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActivityConfigTempDao;
import com.chinacreator.video.mapper.SysDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.SHAUtil;
import com.chinacreator.video.util.URLUtil;
import com.chinacreator.video.util.UsermobUtil;

@Component
public class SmsMango2009Service {
	
	private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");
	
	//只能领取此活动会员
	private static String  activetype = "mg2009";
	
	@Autowired
	private SysDao sysdao;
	
	@Autowired
	private ActivityConfigTempDao activityConfigDao;
	
	@Autowired
	private RedisService redisService;
	
	@Autowired
	private ComonService comonService;

	
	/**
	 * 并发请求
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		try {
			long count = redisService.incr("mangosms_" + vipInfo.getActivetype() + "_" + vipInfo.getUserid());
			if (count > 1) {
				throw new BussinessException("8001", "并发请求，请稍后再试");
			}
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw e;
			}else{
				ErrorLOGGER.error("", e);  
			}
		}
	}
	
	
	/**
	 * 活动相关信息校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkMango(VipInfo vipInfo) throws BussinessException {
		if(!activetype.equals(vipInfo.getActivetype())){
			throw new BussinessException("9083", "无效活动");
		}
	}
	
	
	public void checkActivity(VipInfo vipInfo) throws BussinessException {
		if(StringUtils.isEmpty(vipInfo.getActivetype())) {
			throw new BussinessException("9083", "Activetype无效");
		}
		
		ActivityConfigBean activityConfigBean = activityConfigDao.getConfigTemp(vipInfo.getActivetype());
		if(activityConfigBean == null || !activityConfigBean.getStatus().equals("0") || 
				NumberUtils.toLong(sysdao.getSysTime())>NumberUtils.toLong(activityConfigBean.getEndtime()) || 
				NumberUtils.toLong(sysdao.getSysTime())<NumberUtils.toLong(activityConfigBean.getBegintime())) {
			throw new BussinessException("9083", "该活动无效");
		}
		
		vipInfo.setCpid(activityConfigBean.getCpid());
		vipInfo.setSpid(activityConfigBean.getSpid());
	}
	
	public void checkSign(VipInfo vipInfo) throws BussinessException {
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		String pwd = vipInfo.getCpid();
		String localSign = MD5.MD5Encode(vipInfo.getActivetype() + vipInfo.getUserid() + vipInfo.getTimestamp()+vipInfo.getRemark() + pwd);
		if (!localSign.equals(vipInfo.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}
	
	public void checkMob(VipInfo vipInfo) throws BussinessException {
		String userid = vipInfo.getUserid();
		if(StringUtils.isEmpty(userid)) {
			throw new BussinessException("9054", "userid不能为空");
		}
		
		String pwd = vipInfo.getCpid();
		try {
			userid = DesUtil.decode(userid, pwd);
		}catch(Exception e) {
			throw new BussinessException("9056", "userid加密错误");
		}
		if(!UsermobUtil.isValid(userid)) {
			throw new BussinessException("9055", "userid不正确");
		}
		vipInfo.setUserid(userid);
	}
	
	
	/**
	 * 验证分享的5个号码是否正常
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkShare(VipInfo vipInfo) throws BussinessException {
		String data = vipInfo.getRemark();
		if(data == null || "".equals(data.trim())) {
			throw new BussinessException("9084", "分享号码不能为空");
		}
		
		String [] list = null;
		try {
			list = data.split(",");
		} catch (Exception e) {
			throw new BussinessException("9084", "分享号码异常");
		}
		
		if(list.length>5) {
			throw new BussinessException("9084", "分享号码不能超过5条");
		}
		
		for(String mob :list) {
			if(!UsermobUtil.isValid(mob)) {
				throw new BussinessException("9084", "分享号码不正确:"+mob);
			}
		}
		
		for(int i=0;i<list.length;i++) {
			String bo = list[i];
			for(int j=i+1;j<list.length;j++) {
				if(bo.equals(list[j])) {
					throw new BussinessException("9084", "分享号码重复:"+bo);
				}
			}
		}
		
	}
	
	
	/**
	 * 
	 * 验证数量
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkCount(VipInfo vipInfo) throws BussinessException {
		int count = 0;
		try {
			count = Integer.parseInt(vipInfo.getSpid());
		} catch (Exception e) {
			throw new BussinessException("9083", "活动配置异常，分享次数配置非法");
		}
		
		int real = 0;
		try {
			real = activityConfigDao.queryMango2009ShareCount(vipInfo.getUserid());
		} catch (Exception e) {
			e.printStackTrace();
			throw new BussinessException("9084", "分享次数查询异常");
		}
		
		int zb = vipInfo.getRemark().split(",").length;
		
		if(real+zb > count) {
			throw new BussinessException("9084", "分享次数达到上限");
		}
	}
	
	/**
	 * 领取资格查询校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkSms(VipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);//并发请求
		this.checkMango(vipInfo);//判断活动代码
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkActivity(vipInfo);
		this.checkSign(vipInfo);
		this.checkMob(vipInfo);
		
		this.checkShare(vipInfo);
		this.checkCount(vipInfo);
	}
	
	
	/**
	 * 会员领取
	 * 
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void giveDoing(VipInfo vipInfo) throws BussinessException {
		String [] list = vipInfo.getRemark().split(",");
		
		String result="";
		
		for(String mob:list) {
			try {
				String smsid="527419384";
				String pwd= "W23nV5zl";
				String contentid= "20200821005";
				String vars = "{\"userid\":\""+vipInfo.getUserid()+"\"}";
				String url ="http://111.206.133.54/smsbusi/sms/send";
				result = result +","+this.sendSMS(url, smsid, pwd, mob, contentid,vars);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		/**
		 * 保存分享日志
		 */
		
		try {
			activityConfigDao.insertMango2009ShareLog(vipInfo.getActivetype(), vipInfo.getUserid(), vipInfo.getRemark(), list.length+"",result);
		} catch (Exception e) {
			throw new BussinessException("9084", "分享日志保存异常");
		}
	}
	
	/**
	 * 发送短信
	 * 
	 * @param usermob
	 * @param content
	 * @return
	 */
	private String sendSMS (String dealurl,String smsid,String key, String usermob, String contentid,String vars){
		try {
			String timestamp = System.currentTimeMillis()/1000+"";
			String userid =  DesUtil.encode(usermob,  key);
			String sign = SHAUtil.shaEncode(smsid+userid+timestamp+vars+contentid+key).toLowerCase();
			
			userid = URLEncoder.encode(userid,"utf-8");
			vars = URLEncoder.encode(vars,"utf-8");
			String url = "?userid="+userid+"&smsid="+smsid+"&timestamp="+timestamp+"&sign="+sign+"&vars="+vars+"&contentid="+contentid;
			String resp = URLUtil.get(dealurl+url);
			
			System.out.println(url+","+resp);
			return resp;
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
}
