package com.chinacreator.video.service;


import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.chinacreator.video.bean.ActivityAiqy7Bean;
import com.chinacreator.video.bean.ActivityAiqy7OrderRelation;
import com.chinacreator.video.bean.ActivityConfigBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActiveAiqy7Dao;
import com.chinacreator.video.mapper.ActivityConfigDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.SHAUtil;
import com.chinacreator.video.util.StrUtil;
import com.chinacreator.video.util.URLUtil;
import com.chinacreator.video.util.VipChangeUtil;

@Component
public class VipAiqy7Service {
	
	private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");
	
	//只能领取此活动会员
	private static String  activetype = "30";
	
	private static String vipid="21592";
	private static String vippwd="7aqy2aty";
	private static String orderid_p = "2020aiqy7_";
	
	@Autowired
	private RedisService redisService;
	
	@Autowired
	private ComonService comonService;

	@Autowired
	private MobService mobService;

	@Autowired
	private ActiveAiqy7Dao activeDao;
	
	@Autowired
	private ActivityConfigDao activityConfigDao;

	@Autowired
	VipChangeUtil vipChangeUtil;
	/**
	 * 并发请求
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		try {
			long count = redisService.incr("joinvip_" + vipInfo.getActivetype() + "_" + vipInfo.getUserid());
			if (count > 1) {
				throw new BussinessException("8001", "并发请求，请稍后再试");
			}
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw e;
			}else{
				ErrorLOGGER.error("", e);  
			}
		}
	}
	
	
	/**
	 * 活动相关信息校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkAiqy7(VipInfo vipInfo) throws BussinessException {
		if(!activetype.equals(vipInfo.getActivetype())){
			throw new BussinessException("9083", "无效活动");
		}
	}
	
	
	/**
	 * 判断是否已经过了活动期
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkActivity(VipInfo vipInfo) throws BussinessException {
		if(StringUtils.isEmpty(vipInfo.getActivetype())) {
			throw new BussinessException("9083", "无效活动");
		}
		
		ActivityConfigBean config = activityConfigDao.getConfig(vipInfo.getActivetype());
		if(config == null || !"0".equals(config.getStatus()) || NumberUtils.toLong(activeDao.getSysTime()) < NumberUtils.toLong(config.getBegintime())) {
			throw new BussinessException("9083", "无效活动");
		}
		
		/**
		 * 过了参加活动的时间
		 */
		if(NumberUtils.toLong(activeDao.getSysTime())>NumberUtils.toLong(config.getEndtime()) ) {
			vipInfo.setActivetime(false);
			
			try {
				SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
				Calendar cal = Calendar.getInstance();
				cal.setTime(sdf.parse(config.getEndtime().substring(0,6)+"01000000"));
				cal.add(Calendar.MONTH, 3);
				cal.add(Calendar.SECOND,-1);
				if( NumberUtils.toLong(activeDao.getSysTime()) > NumberUtils.toLong(sdf.format(cal.getTime())) ) {
					throw new BussinessException("9083", "无效活动");
				}
			} catch (Exception e) {
				throw new BussinessException("9083", "无效活动");
			}
		}else {
			vipInfo.setActivetime(true);
		}
		
		vipInfo.setCpid(config.getCpid());
		vipInfo.setSpid(config.getSpid());
		vipInfo.setBegintime(config.getBegintime());
		vipInfo.setEndtime(config.getEndtime());
	}
	
	
	
	/**
	 * 校验活动参与资格
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void queryJoin(VipInfo vipInfo) throws BussinessException {
		ActivityAiqy7Bean bean = activeDao.findByUserid(vipInfo.getUserid());
		if(bean == null) {//之前没有参加过活动并领取过会员
			//查询订购关系
			ActivityAiqy7OrderRelation relation = activeDao.findOrderRelationByUserid(vipInfo.getUserid());
			if(relation == null ) {
				throw new BussinessException("8601", "非活动用户");
			}else if(!StrUtil.isNotBlank(relation.getOrdertime())){
				throw new BussinessException("8600", "数据异常");
			}else if( NumberUtils.toLong(relation.getOrdertime()) < NumberUtils.toLong(vipInfo.getBegintime())){
				throw new BussinessException("8601", "非活动用户");
			}else if( NumberUtils.toLong(relation.getOrdertime()) > NumberUtils.toLong(vipInfo.getEndtime())) {
				throw new BussinessException("8601", "非活动用户");
			}else if( NumberUtils.toLong(relation.getOrdertime()) > NumberUtils.toLong(activeDao.getSysTime()) ) {
				throw new BussinessException("8609", "非活动用户");
			}else {
				
				if("0".equals(relation.getStatus())) {
					/**
					 * 订购状态可以进行领取
					 */
					return;
					
				}else {
					if(!StrUtil.isNotBlank(relation.getCanceltime())) {
						throw new BussinessException("8600", "数据异常");
					}else if(activeDao.getSysTime().substring(0,6).contentEquals(relation.getCanceltime().substring(0,6))) {
						/**
						 * 退订时间当月可以进行领取
						 */
						return;
						
					}else {
						activeDao.insert(vipInfo.getUserid(), relation.getCpid(),relation.getSpid(),"1", vipInfo.getActivetype(), relation.getOrdertime(), relation.getCanceltime());
						
						throw new BussinessException("8601", "非活动用户");
					}
				}
			}
		
		}else {//之前已经参加过活动，并领取过会员
			if("9".equals(bean.getStatus())) {
				throw new BussinessException("8600", "数据异常");
			}
			
			//本月是否已经领取过会员
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
			Date lastVipTime = bean.getViptime();
			if(lastVipTime != null && sdf.format(new Date()).equals(sdf.format(lastVipTime))) {
				throw new BussinessException("8602", "本月已领取");
			}
			
			//判断是否已经退订，并结束活动
			if(!"0".equals(bean.getStatus()) && !sdf.format(new Date()).equals(bean.getCanceltime().substring(0,6))) {
				throw new BussinessException("8603", "已退出活动");
			}
			
			if(!StrUtil.isNotBlank(bean.getOrdertime())){
				throw new BussinessException("8600", "数据异常");
			}
			
			//本月是否还在活动3个月期内
			try {
				SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
				Calendar cal = Calendar.getInstance();
				cal.setTime(sdf2.parse(bean.getOrdertime().substring(0,6)+"01000000"));
				cal.add(Calendar.MONTH, 3);
				cal.add(Calendar.SECOND,-1);
				if( NumberUtils.toLong(activeDao.getSysTime()) > NumberUtils.toLong(sdf2.format(cal.getTime())) ) {
					throw new BussinessException("8604", "活动已结束");
				}
			} catch (Exception e) {
				throw new BussinessException("8604", "活动已结束");
			}
			
			//查询订购关系
			ActivityAiqy7OrderRelation relation = activeDao.findOrderRelationByUserid(vipInfo.getUserid());
			if(relation == null ) {
				throw new BussinessException("8601", "已退出活动");
			}else if(!StrUtil.isNotBlank(relation.getOrdertime())){
				throw new BussinessException("8600", "数据异常");
			}else if(relation.getOrdertime().equals(bean.getOrdertime())) {//订购时间，与第一次领取时相同
				if(!"0".equals(bean.getStatus()) && sdf.format(new Date()).equals(bean.getCanceltime().substring(0,6))) {
					/**
					 * 当前退订的可以领取会员
					 */
					return;
				}else if(!"0".equals(bean.getStatus())) {
					throw new BussinessException("8608", "已退出活动");
				}
				
				if("0".equals(relation.getStatus())) {
					/**
					 * 当前用户在活动期间订购了，一直没有退订，可以参加活动
					 */
					return;
					
				}else {
					if(!StrUtil.isNotBlank(relation.getCanceltime())){
						throw new BussinessException("8600", "数据异常");
					}
					
					if(sdf.format(new Date()).equals(relation.getCanceltime().substring(0,6))) {
						/**
						 * 本月已经退订，本月可以领取会员，并更新资格表
						 */
						return;
						
					}else{
						/**
						 * 本月之前已经退订，不可以领取会员，但要更新资格表
						 */
						activeDao.getNotVipUpdate(vipInfo.getUserid(), "1", relation.getCanceltime());
						
						throw new BussinessException("8606", "已退出活动");
					}
				}
			}else {//订购时间，与第一次领取时不相同
				if(!"0".equals(bean.getStatus()) && sdf.format(new Date()).equals(bean.getCanceltime().substring(0,6))) {
					/**
					 * 当前退订的可以领取会员
					 */
					return;
				}else if(!"0".equals(bean.getStatus())) {
					throw new BussinessException("8608", "已退出活动");
				}
				
				
				List<String> list = activeDao.findCanceltimeFromLog(vipInfo.getUserid());
				if(list == null || list.size()<=0) {
					/**
					 * 订购时间不一样了，但又没有退订日志
					 */
					activeDao.getNotVipUpdate(vipInfo.getUserid(), "9", "");
					throw new BussinessException("8600", "数据异常");
				}else {
					String next = null;
					for(String ctime:list) {
						if(NumberUtils.toLong(bean.getOrdertime()) < NumberUtils.toLong(ctime) ) {
							next = ctime;
						}else {
							break;
						}
					}
					if(next == null) {
						/**
						 * 订购时间不一样了，但又没有退订日志
						 */
						activeDao.getNotVipUpdate(vipInfo.getUserid(), "9", "");
						throw new BussinessException("8600", "数据异常");
					}else {
						
						if(sdf.format(new Date()).equals(next.substring(0,6))) {
							/**
							 * 已经退订，退订当前可以领取会员
							 */
							return;
						}else {
							/**
							 * 已经退订，且不是当月退订，所以要记录更新时间
							 */
							activeDao.getNotVipUpdate(vipInfo.getUserid(), "1", next);
							
							throw new BussinessException("8608", "已退出活动");
						}
						
					}
				}
			}
		}
	}
	
	
	/**
	 * 领取资格查询校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVipQuery(VipInfo vipInfo) throws BussinessException {
		this.checkAiqy7(vipInfo);//判断活动代码
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkActivity(vipInfo);
		comonService.checkQueryVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);
	}
	
	/**
	 * 领取校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVip(VipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);//并发请求
		this.checkAiqy7(vipInfo);//判断活动代码
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkActivity(vipInfo);
		comonService.checkQueryVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);
	}
	
	
	public void giveJoin(VipInfo vipInfo) throws BussinessException {
		ActivityAiqy7Bean bean = activeDao.findByUserid(vipInfo.getUserid());
		if(bean == null) {//之前没有参加过活动并领取过会员
			//查询订购关系
			ActivityAiqy7OrderRelation relation = activeDao.findOrderRelationByUserid(vipInfo.getUserid());
			if(relation == null ) {
				throw new BussinessException("8601", "非活动用户");
			}else if(!StrUtil.isNotBlank(relation.getOrdertime())){
				throw new BussinessException("8600", "数据异常");
			}else if( NumberUtils.toLong(relation.getOrdertime()) < NumberUtils.toLong(vipInfo.getBegintime())){
				throw new BussinessException("8601", "非活动用户");
			}else if( NumberUtils.toLong(relation.getOrdertime()) > NumberUtils.toLong(vipInfo.getEndtime())) {
				throw new BussinessException("8601", "非活动用户");
			}else if( NumberUtils.toLong(relation.getOrdertime()) > NumberUtils.toLong(activeDao.getSysTime()) ) {
				throw new BussinessException("8609", "非活动用户");
			}else {
				
				if("0".equals(relation.getStatus())) {
					/**
					 * 订购状态可以进行领取
					 */
					activeDao.insert(vipInfo.getUserid(), relation.getCpid(),relation.getSpid(),"0", vipInfo.getActivetype(), relation.getOrdertime(), "");
					this.getVip(vipInfo);
					activeDao.getVipUpdate(vipInfo.getUserid(), "0", "");
					
				}else {
					if(!StrUtil.isNotBlank(relation.getCanceltime())) {
						throw new BussinessException("8600", "数据异常");
					}else if(activeDao.getSysTime().substring(0,6).contentEquals(relation.getCanceltime().substring(0,6))) {
						/**
						 * 退订时间当月可以进行领取
						 */
						activeDao.insert(vipInfo.getUserid(), relation.getCpid(),relation.getSpid(),"1", vipInfo.getActivetype(), relation.getOrdertime(), relation.getCanceltime());
						this.getVip(vipInfo);
						activeDao.getVipUpdate(vipInfo.getUserid(), "1", relation.getCanceltime());
						
					}else {
						activeDao.insert(vipInfo.getUserid(), relation.getCpid(),relation.getSpid(),"1", vipInfo.getActivetype(), relation.getOrdertime(), relation.getCanceltime());
						
						throw new BussinessException("8601", "非活动用户");
					}
				}
			}
		
		}else {//之前已经参加过活动，并领取过会员
			if("9".equals(bean.getStatus())) {
				throw new BussinessException("8600", "数据异常");
			}
			
			//本月是否已经领取过会员
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
			Date lastVipTime = bean.getViptime();
			if(lastVipTime != null && sdf.format(new Date()).equals(sdf.format(lastVipTime))) {
				throw new BussinessException("8602", "本月已领取");
			}
			
			//判断是否已经退订，并结束活动
			if(!"0".equals(bean.getStatus()) && !sdf.format(new Date()).equals(bean.getCanceltime().substring(0,6))) {
				throw new BussinessException("8603", "已退出活动");
			}
			
			if(!StrUtil.isNotBlank(bean.getOrdertime())){
				throw new BussinessException("8600", "数据异常");
			}
			
			//本月是否还在活动3个月期内
			try {
				SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
				Calendar cal = Calendar.getInstance();
				cal.setTime(sdf2.parse(bean.getOrdertime().substring(0,6)+"01000000"));
				cal.add(Calendar.MONTH, 3);
				cal.add(Calendar.SECOND,-1);
				if( NumberUtils.toLong(activeDao.getSysTime()) > NumberUtils.toLong(sdf2.format(cal.getTime())) ) {
					throw new BussinessException("8604", "活动已结束");
				}
			} catch (Exception e) {
				throw new BussinessException("8604", "活动已结束");
			}
			
			//查询订购关系
			ActivityAiqy7OrderRelation relation = activeDao.findOrderRelationByUserid(vipInfo.getUserid());
			if(relation == null ) {
				throw new BussinessException("8601", "已退出活动");
			}else if(!StrUtil.isNotBlank(relation.getOrdertime())){
				throw new BussinessException("8600", "数据异常");
			}else if(relation.getOrdertime().equals(bean.getOrdertime())) {//订购时间，与第一次领取时相同
				/************************************************************************/
				if(!"0".equals(bean.getStatus()) && sdf.format(new Date()).equals(bean.getCanceltime().substring(0,6))) {
					/**
					 * 当前退订的，可以领取会员
					 */
					this.getVip(vipInfo);
					activeDao.getVipUpdate(vipInfo.getUserid(), "1", bean.getCanceltime());
					return;
					
				}else if(!"0".equals(bean.getStatus())) {
					throw new BussinessException("8608", "已退出活动");
				}
				/************************资格表中，已经退订，不会再处理**************************/
				
				if("0".equals(relation.getStatus())) {
					/**
					 * 当前用户在活动期间订购了，一直没有退订，可以参加活动
					 */
					this.getVip(vipInfo);
					activeDao.getVipUpdate(vipInfo.getUserid(), "0", "");
					
				}else {
					if(!StrUtil.isNotBlank(relation.getCanceltime())){
						throw new BussinessException("8600", "数据异常");
					}
					
					if(sdf.format(new Date()).equals(relation.getCanceltime().substring(0,6))) {
						/**
						 * 本月已经退订，本月可以领取会员，并更新资格表
						 */
						this.getVip(vipInfo);
						activeDao.getVipUpdate(vipInfo.getUserid(), "1", relation.getCanceltime());
						
					}else{
						/**
						 * 本月之前已经退订，不可以领取会员，但要更新资格表
						 */
						activeDao.getNotVipUpdate(vipInfo.getUserid(), "1", relation.getCanceltime());
						
						throw new BussinessException("8606", "已退出活动");
					}
				}
			}else {//订购时间，与第一次领取时不相同
				/************************************************************************/
				if(!"0".equals(bean.getStatus()) && sdf.format(new Date()).equals(bean.getCanceltime().substring(0,6))) {
					/**
					 * 当前退订的，可以领取会员
					 */
					this.getVip(vipInfo);
					activeDao.getVipUpdate(vipInfo.getUserid(), "1", bean.getCanceltime());
					return;
					
				}else if(!"0".equals(bean.getStatus())) {
					throw new BussinessException("8608", "已退出活动");
				}
				/************************资格表中，已经退订，不会再处理**************************/
				
				List<String> list = activeDao.findCanceltimeFromLog(vipInfo.getUserid());
				if(list == null || list.size()<=0) {
					/**
					 * 订购时间不一样了，但又没有退订日志
					 */
					activeDao.getNotVipUpdate(vipInfo.getUserid(), "9", "");
					throw new BussinessException("8600", "数据异常");
				}else {
					String next = null;
					for(String ctime:list) {
						if(NumberUtils.toLong(bean.getOrdertime()) < NumberUtils.toLong(ctime) ) {
							next = ctime;
						}else {
							break;
						}
					}
					if(next == null) {
						/**
						 * 订购时间不一样了，但又没有退订日志
						 */
						activeDao.getNotVipUpdate(vipInfo.getUserid(), "9", "");
						throw new BussinessException("8600", "数据异常");
					}else {
						
						if(sdf.format(new Date()).equals(next.substring(0,6))) {
							/**
							 * 已经退订，退订当前可以领取会员
							 */
							this.getVip(vipInfo);
							activeDao.getVipUpdate(vipInfo.getUserid(), "1", next);
						}else {
							/**
							 * 已经退订，且不是当月退订，所以要记录更新时间
							 */
							activeDao.getNotVipUpdate(vipInfo.getUserid(), "1", next);
							
							throw new BussinessException("8608", "已退出活动");
						}
						
					}
				}
			}
		}
	}
	
	
	/**
	 * 领取会员
	 * 
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void getVip(VipInfo vipInfo) throws BussinessException {
		String month = new SimpleDateFormat("yyyyMM").format(new Date());
		
		String orderid = orderid_p + month+"01"+vipInfo.getUserid();
		
		//领取会员
		vipChangeUtil.getVip(vipid, vipInfo.getUserid(), vipInfo.getActivetype(), vippwd, "4", vipInfo.getUserid(), orderid, null , vipInfo.getType()+"");
		
		activeDao.insertVip(vipInfo.getUserid(), month,"0", vipInfo.getActivetype(), vipid, orderid);
		
		this.sendSMS(vipInfo.getUserid());
	}
	
	
	
	/**
	 * 发送短信
	 * 
	 * @param userid
	 * @return
	 */
	private String sendSMS (String userid){
		try {
			String smsid="5003102";
			String key= "soews92d";
			String content = "【中国联通】7日爱奇艺黄金VIP会员已领取至手机号码"+userid+"，请在爱奇艺登录手机号码使用。";
			
			String timestamp = System.currentTimeMillis()/1000+"";
			String usermob =  DesUtil.encode(userid,  key);
			String sign = SHAUtil.shaEncode(smsid+usermob+timestamp+content+key).toLowerCase();
			
			usermob = URLEncoder.encode(usermob,"utf-8");
			String contentStr = URLEncoder.encode(content,"utf-8");
			String url = "http://111.206.133.54/smsbusi/sms/send?userid="+usermob+"&smsid="+smsid+"&timestamp="+timestamp+"&content="+contentStr+"&sign="+sign;
			String resp = URLUtil.get(url);
			
			System.out.println(url+","+resp);
			return resp;
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
}
