package com.chinacreator.video.service;


import java.net.URLEncoder;

import org.apache.commons.lang.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.chinacreator.video.bean.ActivityConfigBean;
import com.chinacreator.video.bean.CpSpBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActiveAiqyHBDao;
import com.chinacreator.video.mapper.ActivityConfigDao;
import com.chinacreator.video.mapper.CpSpDao;
import com.chinacreator.video.mapper.SysDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.SHAUtil;
import com.chinacreator.video.util.StrUtil;
import com.chinacreator.video.util.URLUtil;
import com.chinacreator.video.util.VipChangeUtil;

@Component
public class VipAiqyHBService {
	
	private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");
	
	//只能领取此活动会员
	private static String  activetype = "31";
	
	private static String vipid="21593";
	private static String vippwd="os2sena0";
	private static String orderid_p = "2020aiqy7hb_";
	
	@Autowired
	private RedisService redisService;
	
	@Autowired
	private ComonService comonService;

	@Autowired
	private MobService mobService;
	
	@Autowired
	private SysDao sysdao;
	
	@Autowired
	private CpSpDao CpSpDao;

	@Autowired
	private ActiveAiqyHBDao activeDao;
	
	@Autowired
	private ActivityConfigDao activityConfigDao;

	@Autowired
	VipChangeUtil vipChangeUtil;
	/**
	 * 并发请求
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		try {
			long count = redisService.incr("joinvip_" + vipInfo.getActivetype() + "_" + vipInfo.getUserid());
			if (count > 1) {
				throw new BussinessException("8001", "并发请求，请稍后再试");
			}
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw e;
			}else{
				ErrorLOGGER.error("", e);  
			}
		}
	}
	
	
	/**
	 * 活动相关信息校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkAiqy7(VipInfo vipInfo) throws BussinessException {
		if(!activetype.equals(vipInfo.getActivetype())){
			throw new BussinessException("9083", "无效活动");
		}
	}
	
	public void checkActivity(VipInfo vipInfo) throws BussinessException {
		System.out.println(JSON.toJSONString(vipInfo));
		if(StringUtils.isEmpty(vipInfo.getActivetype())) {
			throw new BussinessException("9083", "Activetype无效");
		}
		ActivityConfigBean activityConfigBean = activityConfigDao.getConfig(vipInfo.getActivetype());
		System.out.println(JSON.toJSONString(activityConfigBean));
		if(activityConfigBean == null || !activityConfigBean.getStatus().equals("0") || NumberUtils.toLong(sysdao.getSysTime())>NumberUtils.toLong(activityConfigBean.getEndtime()) || NumberUtils.toLong(sysdao.getSysTime())<NumberUtils.toLong(activityConfigBean.getBegintime())) {
			throw new BussinessException("9083", "该活动无效");
		}
		vipInfo.setBegintime(activityConfigBean.getBegintime());
		vipInfo.setEndtime(activityConfigBean.getEndtime());
		
		
		if(!StrUtil.isNotBlank(activityConfigBean.getChannel())) {
			throw new BussinessException("8501", "活动订购渠道配置异常");
		}
		
		if(!StrUtil.isNotBlank(vipInfo.getChannel())) {
			throw new BussinessException("8502", "channel参数为空");
		}
		
		String [] Channels = activityConfigBean.getChannel().split(",");
		if(!StrUtil.have(Channels, vipInfo.getChannel())) {
			throw new BussinessException("8503", "此订购渠道不能参加活动");
		}
//		if(!vipInfo.getChannel().equals(activityConfigBean.getChannel())) {
//			throw new BussinessException("8503", "此订购渠道不能参加活动");
//		}
		
		if(!StrUtil.isNotBlank(vipInfo.getCpid())) {
			throw new BussinessException("8504", "cpid参数为空");
		}
		
		if(!StrUtil.isNotBlank(vipInfo.getSpid())) {
			throw new BussinessException("8505", "spid参数为空");
		}
		
		if(!StrUtil.isNotBlank(activityConfigBean.getCpid())) {
			throw new BussinessException("8506", "活动CPID配置异常");
		}
		
		if(!StrUtil.isNotBlank(activityConfigBean.getSpid())) {
			throw new BussinessException("8507", "活动SPID配置异常");
		}
		
		String [] cpids = activityConfigBean.getCpid().split(",");
		String [] spids = activityConfigBean.getSpid().split(",");
		if(!StrUtil.have(cpids, vipInfo.getCpid())) {
			throw new BussinessException("8508", "cpid不能参加此活动");
		}
		if(!StrUtil.have(spids, vipInfo.getSpid())) {
			throw new BussinessException("8509", "spid不能参加此活动");
		}
		
	}
	
	public void checkVipSign(VipInfo vipInfo) throws BussinessException {
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(vipInfo.getCpid(), vipInfo.getSpid());
		String pwd = cpSpBean.getPwd();
		String localSign = MD5.MD5Encode(vipInfo.getUserid() + vipInfo.getChannel() + vipInfo.getCpid() + vipInfo.getSpid() + pwd + vipInfo.getTimestamp());
		if (!localSign.equals(vipInfo.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}
	
	public void checkJoinVip(VipInfo vipInfo) throws BussinessException {
		int joincount = activeDao.getJoinRecord(vipInfo.getUserid(),vipInfo.getCpid(), vipInfo.getSpid());
		if(joincount > 0) {
			throw new BussinessException("8510", "您已经领取会员，不能重复领取");
		}
	}
	
	public void checkOrderRelation(VipInfo vipInfo) throws BussinessException {
		int ordercount = activeDao.queryOrderRelation(vipInfo.getUserid(),vipInfo.getCpid(), vipInfo.getSpid(), vipInfo.getChannel(),vipInfo.getBegintime(),vipInfo.getEndtime());
		if(ordercount <= 0) {
			throw new BussinessException("8513", "您没有领取资格，不能领取会员");
		}
	}
	
	
	/**
	 * 领取校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVip(VipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);//并发请求
		this.checkAiqy7(vipInfo);//判断活动代码
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkActivity(vipInfo);
		this.checkVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);
		
		this.checkJoinVip(vipInfo);
		this.checkOrderRelation(vipInfo);
	}
	
	
	public void giveJoin(VipInfo vipInfo) throws BussinessException {
		String orderid = orderid_p  +vipInfo.getUserid() + vipInfo.getSpid();
		int i=activeDao.insert(vipInfo.getUserid(), vipInfo.getCpid(), vipInfo.getSpid(), "1", vipInfo.getActivetype());
		if(i>0) {
			
			try {
				//领取会员
				vipChangeUtil.getVip(vipid, vipInfo.getUserid(), vipInfo.getActivetype(), vippwd, "4", vipInfo.getUserid(), orderid, null ,vipInfo.getType()+"");
			
			} catch (BussinessException e) {
				activeDao.deteleJoining(vipInfo.getUserid(), vipInfo.getCpid(), vipInfo.getSpid());
				
				throw new BussinessException("8511", "会员领取失败，请稍后再试");
			}
			
			activeDao.joinSucceed(vipInfo.getUserid(), vipInfo.getCpid(), vipInfo.getSpid(), "0");
			
			this.sendSMS(vipInfo.getUserid());
			
		}else {
			throw new BussinessException("8512", "会员领取失败，请稍后再试");
		}
	}
	
	
	
	/**
	 * 发送短信
	 * 
	 * @param userid
	 * @return
	 */
	private String sendSMS (String userid){
		try {
			String smsid="5003102";
			String key= "soews92d";
			String content = "【中国联通】7日爱奇艺黄金VIP会员已领取至手机号码"+userid+"，请在爱奇艺登录手机号码使用。";
			
			String timestamp = System.currentTimeMillis()/1000+"";
			String usermob =  DesUtil.encode(userid,  key);
			String sign = SHAUtil.shaEncode(smsid+usermob+timestamp+content+key).toLowerCase();
			
			usermob = URLEncoder.encode(usermob,"utf-8");
			String contentStr = URLEncoder.encode(content,"utf-8");
			String url = "http://111.206.133.54/smsbusi/sms/send?userid="+usermob+"&smsid="+smsid+"&timestamp="+timestamp+"&content="+contentStr+"&sign="+sign;
			String resp = URLUtil.get(url);
			
			System.out.println(url+","+resp);
			return resp;
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	
}
