package com.chinacreator.video.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.chinacreator.video.bean.ContinueBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.VipGiveDao;
import com.chinacreator.video.redis.RedisService;

@Component
public class VipControlMultiService {

	@Autowired
	private RedisService redisService;

	@Autowired
	private VipGiveDao vipGiveDao;

	public void canGiveVip(VipInfo vipInfo) throws BussinessException {
		try {
			checkRedis(vipInfo);
			checkGiveed(vipInfo);
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw e;
			}
		}
	}
	
	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		long count = redisService.incr("vip_multi_" +vipInfo.getActivetype()+ "_" + vipInfo.getCpid() + "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid());
		if (count > 1) {
			throw new BussinessException("8003", "赠送会员并发请求，请稍后再试");
		}
	}

	public void checkGiveed(VipInfo vipInfo) throws BussinessException {
		String countnum = this.vipGiveDao.getVipGiveInfoMulti(vipInfo.getUserid(),vipInfo.getActivetype());
		int count = Integer.parseInt(countnum);
		if(count>=vipInfo.getGivecount()){
			throw new BussinessException("8005", "会员已经领取，或没有会员可以领取");
		}
	}
	
	
	
	public void canGiveVip(ContinueBean vipInfo) throws BussinessException {
		try {
			checkRedis(vipInfo);
			checkGiveed(vipInfo);
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw e;
			}
		}
	}
	
	public void checkRedis(ContinueBean vipInfo) throws BussinessException {
		long count = redisService.incr("vip_continue_" +vipInfo.getActivityType()+ "_" + vipInfo.getCpid() + "_" + vipInfo.getSpid()+"_" + vipInfo.getUserid());
		if (count > 1) {
			throw new BussinessException("8003", "赠送会员并发请求，请稍后再试");
		}
	}

	public void checkGiveed(ContinueBean vipInfo) throws BussinessException {
		String countnum = this.vipGiveDao.getVipGiveInfoMulti(vipInfo.getUserid(),vipInfo.getActivityType());
		int count = Integer.parseInt(countnum);
		if(count>=vipInfo.getGivecount()){
			throw new BussinessException("8005", "会员已经领取，或没有会员可以领取");
		}
	}
	
	/**
	 * 是否已经领取成功会员
	 * 
	 * @param vipInfo
	 * @return
	 * @throws BussinessException
	 */
	public boolean isGiveOver(VipInfo vipInfo) throws BussinessException {
//		String countnum = this.vipGiveDao.getVipGiveInfoMulti(vipInfo.getUserid(),vipInfo.getActivetype());
		String countnum = this.vipGiveDao.getVipGiveInfoMultiFirst(vipInfo.getUserid(),vipInfo.getActivetype());
		int count = Integer.parseInt(countnum);
		if(count>=vipInfo.getGivecount()){
			return true;
		}
		return false;
	}
	
	public boolean isGiveOver(ContinueBean continueBean) throws BussinessException {
		String countnum = this.vipGiveDao.getVipGiveInfoMulti(continueBean.getUserid(),continueBean.getActivityType());
		int count = Integer.parseInt(countnum);
		if(count>=continueBean.getGivecount()){
			return true;
		}
		return false;
	}

}
