package com.chinacreator.video.service;


import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.chinacreator.video.mapper.*;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.time.DateFormatUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.ActivityBean;
import com.chinacreator.video.bean.ActivityConfigBean;
import com.chinacreator.video.bean.CpSpBean;
import com.chinacreator.video.bean.OrderInfo;
import com.chinacreator.video.bean.UserMsgBean;
import com.chinacreator.video.bean.Vip;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.bean.VipWhiteBean;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.service.handle.GetVipService;
import com.chinacreator.video.thread.VacQueryThread;
import com.chinacreator.video.thread.VacQueryUtil;
import com.chinacreator.video.thread.VipGroupThread;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.MD5;

@Component
public class VipGroupService {

	@Autowired
	private RedisService redisService;

	@Autowired
	private CpSpDao CpSpDao;

	@Autowired
	private ComonService comonService;

	@Autowired
	private MobService mobService;

	@Autowired
	private ActivityConfigService activityService;

	@Autowired
	private GiveVipService giveVipService;

	@Autowired
	private ActiveDao activeDao;

	@Autowired
	private ActiveGroupDao activeGroupDao;

	@Autowired
	private UserMsgDao userMsgDao;

	@Autowired
	private PreOrderDao preOrdeDao;

//	@Autowired
//	private VipDao vipdao;

	@Autowired
	private VipWhiteDao vipWhiteDao;

	@Autowired
	private CtvGiveVipService ctvGiveVipService;

	@Autowired
	private AiqiyiGiveVipService aiqiyiGiveVipService;

	@Autowired
	private GiveVipProvinceCrontrolDao giveVipProvinceCrontrolDao;

	@Autowired
	private AiqiyiGiveMonthVipService aiqiyiGiveMonthVipService;

	@Autowired
	private LeshiGiveVipService leshiGiveVipService;

	@Autowired
	private LeshiNewGiveVipService leshiNewGiveVipService;

	@Autowired
	private MangtvGiveVipService mangtvGiveVipService;

	@Autowired
	private YoukuVipService youkuVipService;

	@Autowired
	private EcbVipService ecbVipService;

	@Autowired
	private ActivityVipConfigService vipConfigService;

	@Autowired
	private ActivityConfigDao activityConfigDao;

	@Autowired
	private GetVipService getVipService;

	@Autowired
	private CapKbxkQryService capKbxkQryService;

	@Autowired
	private DictionaryDao dictionaryDao;

	/**
	 * 复合产品 - 并发控制
	 *
	 * @param orderInfo
	 * @throws BussinessException
	 */
	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		long count = redisService.incr("joinvip_" + vipInfo.getActivetype() + "_"+ vipInfo.getCpid() + "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid());
		if (count > 1) {
			throw new BussinessException("8001", "并发请求，请稍后再试");
		}
	}

	/**
	 * 复合产品 - 并发控制
	 *
	 * @param orderInfo
	 * @throws BussinessException
	 */
	public void checkRedisQuery(VipInfo vipInfo) throws BussinessException {
		long count = redisService.incr("joinvip_query_" + vipInfo.getActivetype() + "_"+ vipInfo.getCpid() + "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid());
		if (count > 1) {
			throw new BussinessException("8001", "并发请求，请稍后再试");
		}
	}

	/**
	 * 会员领取投诉登记 - 并发控制
	 *
	 * @param orderInfo
	 * @throws BussinessException
	 */
	public void checkRedisVipreg(VipInfo vipInfo) throws BussinessException {
		long count = redisService.incr("vipreg_" + vipInfo.getActivetype() + "_"+ vipInfo.getCpid() + "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid());
		if (count > 1) {
			throw new BussinessException("8001", "并发请求，请稍后再试");
		}
	}

	/**
	 * 复合产品-Sign校验
	 *
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void checkGroupVipSign(VipInfo vipInfo) throws BussinessException {
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(vipInfo.getCpid(), vipInfo.getSpid());
		String pwd = cpSpBean.getPwd();
		String data = vipInfo.getActivetype()+vipInfo.getUserid() + vipInfo.getTimestamp()+ pwd;
		String localSign = MD5.MD5Encode(data);
//		System.out.println(sign);
//		System.out.println(localSign);
		if (!localSign.equals(sign)) {
			throw new BussinessException("9054", "signature校验失败");
		}

		vipInfo.setSptype(cpSpBean.getType());
		vipInfo.setPwd(cpSpBean.getPwd());
		vipInfo.setMobtype(cpSpBean.getMobtype());

//		System.out.println("pwd ----"+vipInfo.getPwd());
	}


	/**
	 * 复合产品-请求校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkGroupVip(VipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		this.checkGroupVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);

		//查询可以领取会员的个数
		try {
			ActivityConfigBean config = activityConfigDao.getConfig(vipInfo.getActivetype());
			vipInfo.setGivecount(Integer.parseInt(config.getGivecount()));
		} catch (Exception e) {
			vipInfo.setGivecount(0);
		}
	}

	/**
	 * 复合产品-请求校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkGroupVipQuery(VipInfo vipInfo) throws BussinessException {
//		this.checkRedisQuery(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		this.checkGroupVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);

		//查询可以领取会员的个数
		try {
			ActivityConfigBean config = activityConfigDao.getConfig(vipInfo.getActivetype());
			vipInfo.setGivecount(Integer.parseInt(config.getGivecount()));
		} catch (Exception e) {
			vipInfo.setGivecount(0);
		}
	}

	/**
	 * 会员领取投诉登记
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void vipreg(VipInfo vipInfo) throws BussinessException {
		String cpid = vipInfo.getCpid();
		String spid = vipInfo.getSpid();
		String activetype = vipInfo.getActivetype();
		String remark = vipInfo.getRemark();
		String givetime = vipInfo.getGivetime();

		this.checkRedisVipreg(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		this.checkGroupVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);

		String userid = vipInfo.getUserid();
		try {
			activeGroupDao.insertVipReg(userid==null?"":userid, cpid==null?"":cpid, spid==null?"":spid, activetype==null?"":activetype, remark==null?"":remark, givetime==null?"":givetime);
		} catch (Exception e) {
			throw new BussinessException("8032", "登记失败");
		}
	}


	/**
	 * 复合产品-多会员领取
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void giveGroupVip(VipInfo activityInfo) throws BussinessException {
		//查询活动关系表
		List<ActivityBean> list = activeGroupDao.getActivityInfos(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(list == null || list.size()<=0) {
			OrderInfo orderInfo =preOrdeDao.getOrder(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
			if(orderInfo != null) {
				throw new BussinessException("9088", "预定购用户不能赠送会员");
			}

			//查询本地是否有订购关系
			int i=activeGroupDao.queryOrder(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
			System.out.println("order relation----"+i+","+activityInfo.getUserid()+","+activityInfo.getCpid()+","+activityInfo.getSpid());
			if(i>0){
				throw new BussinessException("3092", "用户有订购，但无会员领取权益");
			}

			throw new BussinessException("9085", "用户无订购不能赠送会员");
		}

		//判断省份
		String province = list.get(0).getProvince();
		for(ActivityBean bean :list){
			if(province == null){
				throw new BussinessException("5029", "省份信息为空");
			}else{
				if(!province.equals(bean.getProvince())){
					throw new BussinessException("5030", "省份信息异常");
				}
			}
		}

		//会员领取省份限制
		String cangive = giveVipProvinceCrontrolDao.queryProvinceCrontrol(activityInfo.getCpid(), activityInfo.getSpid(), province, activityInfo.getActivetype());
		if(!cangive.equals("0")) {
			throw new BussinessException("9098", "该省份会员被限制领取");
		}

		//黑名单
		boolean whiteFlag = true;
		VipWhiteBean vipWhiteBean = vipWhiteDao.getCount("0", activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		if(vipWhiteBean != null && "1".equals(vipWhiteBean.getType())){
			throw new BussinessException("9089", "黑名单用户禁止领取");
		}
		//白名单，跳过领取
		if(vipWhiteBean != null && "0".equals(vipWhiteBean.getType())){
			whiteFlag = false;
		}

		//三户信息-会员赠送限制
		ecbVipService.queryEcb(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(),province, 0, activityInfo.getActivetype());

		//调cap综合用户查询和卡部信控查询并记录
		capKbxkQryService.capKbxkQry(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		//会员领取
		if(whiteFlag){
			List<ActivityBean> vipList = new ArrayList<ActivityBean>();

			int flaggivecount=0;
			boolean flaggive = false;//是否已经领取过会员
			for(ActivityBean bean :list ){
				try {
					//订购状态判断
					if("1".equals(bean.getOrderstatus())) {
						if(NumberUtils.toLong(bean.getEndtime())<NumberUtils.toLong(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss"))){
							throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
						}
					}else if (!"0".equals(bean.getOrderstatus())){
						throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
					}

					//领取状态判断
					if("4".equals(bean.getVipstatus())) {
						throw new BussinessException("9090", "该活动已暂停赠送会员");
					}else if("3".equals(bean.getVipstatus())) {
						flaggive = true;
						flaggivecount++;
						throw new BussinessException("9091", "会员赠送中");
					}else if ("990099".equals(bean.getVipstatus())){
						flaggive = true;
						flaggivecount++;
						throw new BussinessException("9091", "会员赠送中");
					}else if("1".equals(bean.getVipstatus())) {
						flaggive = true;
						flaggivecount++;
						throw new BussinessException("9086", "会员领取失败");
					}else if("0".equals(bean.getVipstatus())) {
						flaggive = true;
						flaggivecount++;
						throw new BussinessException("9087", "会员已领取");
					}else if(!"2".equals(bean.getVipstatus())) {
						throw new BussinessException("9090", "会员领取状态异常");
					}

					//会员领取省份限制
					String cangiveSun = giveVipProvinceCrontrolDao.queryProvinceCrontrol(bean.getCpid(), bean.getSpid(), province, activityInfo.getActivetype());
					if(!cangiveSun.equals("0")) {
						throw new BussinessException("9098", "该省份会员被限制领取");
					}

					//黑名单
					boolean whiteFlagSun = true;
					VipWhiteBean vipWhiteBeanSun = vipWhiteDao.getCount("0", bean.getUserid(),bean.getCpid(),bean.getSpid());
					if(vipWhiteBeanSun != null && "1".equals(vipWhiteBeanSun.getType())){
						throw new BussinessException("9089", "黑名单用户禁止领取");
					}
					//白名单，跳过领取
					if(vipWhiteBeanSun != null && "0".equals(vipWhiteBeanSun.getType())){
						whiteFlagSun = false;
					}

					//子产品会员领取
					if(whiteFlagSun){

						//修改会员领取状态为:3(领取中)
						bean.setOldvipstatus("2");
						bean.setNewvipstatus("3");
						int i =  activeGroupDao.updateVip(bean);
						if(i>0){
							vipList.add(bean);
						}
					}
					System.out.println("activityBean="+JSON.toJSONString(bean));
				} catch (Exception e) {
					if(e instanceof BussinessException) {
						bean.setVipresultcode(((BussinessException) e).getCode());
						bean.setVipresultinfo(e.getMessage());
					}else {
						bean.setVipresultcode("系统忙，请稍后再试");
						bean.setVipresultinfo("8000");
					}
				}
			}


			if(vipList !=null && vipList.size()>0){
				//已经领取过的会员，不能再次领取
				if(flaggive){
					for(ActivityBean bo:vipList){
						try {
							bo.setOldvipstatus("3");
							bo.setNewvipstatus("2");
							activeGroupDao.updateVip(bo);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
					throw new BussinessException("3091", "已经领取过或正在领取中，不能再次领取会员");
				}

				//判断领取数量
				if(activityInfo.getGivecount() < vipList.size()+flaggivecount){
					throw new BussinessException("3090", "已经领取过或正在领取中，不能再次领取会员");
				}

				Thread thread = new VipGroupThread(activityInfo,vipList,this,vipConfigService);
				thread.start();

			}else{
				throw new BussinessException("3090", "已经领取过或正在领取中，不能再次领取会员");
			}
		}

		System.out.println("VipInfo="+JSON.toJSONString(activityInfo));
	}

	/**
	 * 复合产品-多会员领取
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void giveGroupVip2(VipInfo activityInfo) throws BussinessException {
		//查询活动关系表
		List<ActivityBean> list = activeGroupDao.getActivityInfos(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(list == null || list.size()<=0) {
			OrderInfo orderInfo =preOrdeDao.getOrder(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
			if(orderInfo != null) {
				throw new BussinessException("9088", "预定购用户不能赠送会员");
			}

			//查询本地是否有订购关系
			int i=activeGroupDao.queryOrder(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
			System.out.println("order relation----"+i+","+activityInfo.getUserid()+","+activityInfo.getCpid()+","+activityInfo.getSpid());
			if(i>0){
				throw new BussinessException("3092", "用户有订购，但无会员领取权益");
			}

			throw new BussinessException("9085", "用户无订购不能赠送会员");
		}

		//判断省份
		String province = list.get(0).getProvince();
		for(ActivityBean bean :list){
			if(province == null){
				throw new BussinessException("5029", "省份信息为空");
			}else{
				if(!province.equals(bean.getProvince())){
					throw new BussinessException("5030", "省份信息异常");
				}
			}
		}

		//会员领取省份限制
		String cangive = giveVipProvinceCrontrolDao.queryProvinceCrontrol(activityInfo.getCpid(), activityInfo.getSpid(), province, activityInfo.getActivetype());
		if(!cangive.equals("0")) {
			throw new BussinessException("9098", "该省份会员被限制领取");
		}

		//黑名单
		boolean whiteFlag = true;
		VipWhiteBean vipWhiteBean = vipWhiteDao.getCount("0", activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		if(vipWhiteBean != null && "1".equals(vipWhiteBean.getType())){
			throw new BussinessException("9089", "黑名单用户禁止领取");
		}
		//白名单，跳过领取
		if(vipWhiteBean != null && "0".equals(vipWhiteBean.getType())){
			whiteFlag = false;
		}

		//三户信息-会员赠送限制
		ecbVipService.queryEcb(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(),province, 0, activityInfo.getActivetype());

		//调cap综合用户查询和卡部信控查询并记录
		capKbxkQryService.capKbxkQry(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		//会员领取
		if(whiteFlag){
			List<ActivityBean> vipList = new ArrayList<ActivityBean>();

			int flaggivecount=0;
			boolean flaggive = false;//是否已经领取过会员
			for(ActivityBean bean :list ){
				try {
					//订购状态判断
					if("1".equals(bean.getOrderstatus())) {
						if(NumberUtils.toLong(bean.getEndtime())<NumberUtils.toLong(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss"))){
							throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
						}
					}else if (!"0".equals(bean.getOrderstatus())){
						throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
					}

					//领取状态判断
					if("4".equals(bean.getVipstatus())) {
						throw new BussinessException("9090", "该活动已暂停赠送会员");
					}else if("3".equals(bean.getVipstatus())) {
						flaggive = true;
						flaggivecount++;
						throw new BussinessException("9091", "会员赠送中");
					}else if("1".equals(bean.getVipstatus())) {
						flaggive = true;
						flaggivecount++;
						throw new BussinessException("9086", "会员领取失败");
					}else if("0".equals(bean.getVipstatus())) {
						flaggive = true;
						flaggivecount++;
						throw new BussinessException("9087", "会员已领取");
					}else if(!"2".equals(bean.getVipstatus())) {
						throw new BussinessException("9090", "会员领取状态异常");
					}

					//会员领取省份限制
					String cangiveSun = giveVipProvinceCrontrolDao.queryProvinceCrontrol(bean.getCpid(), bean.getSpid(), province, activityInfo.getActivetype());
					if(!cangiveSun.equals("0")) {
						throw new BussinessException("9098", "该省份会员被限制领取");
					}

					//黑名单
					boolean whiteFlagSun = true;
					VipWhiteBean vipWhiteBeanSun = vipWhiteDao.getCount("0", bean.getUserid(),bean.getCpid(),bean.getSpid());
					if(vipWhiteBeanSun != null && "1".equals(vipWhiteBeanSun.getType())){
						throw new BussinessException("9089", "黑名单用户禁止领取");
					}
					//白名单，跳过领取
					if(vipWhiteBeanSun != null && "0".equals(vipWhiteBeanSun.getType())){
						whiteFlagSun = false;
					}

					//子产品会员领取
					if(whiteFlagSun){

						//修改会员领取状态为:3(领取中)
						bean.setOldvipstatus("2");
						bean.setNewvipstatus("3");
						int i =  activeGroupDao.updateVip(bean);
						if(i>0){
							vipList.add(bean);
						}
					}
					System.out.println("activityBean="+JSON.toJSONString(bean));
				} catch (Exception e) {
					if(e instanceof BussinessException) {
						bean.setVipresultcode(((BussinessException) e).getCode());
						bean.setVipresultinfo(e.getMessage());
					}else {
						bean.setVipresultcode("系统忙，请稍后再试");
						bean.setVipresultinfo("8000");
					}
				}
			}


			if(vipList !=null && vipList.size()>0){
//				//已经领取过的会员，不能再次领取
//				if(flaggive){
//					for(ActivityBean bo:vipList){
//						try {
//							bo.setOldvipstatus("3");
//							bo.setNewvipstatus("2");
//							activeGroupDao.updateVip(bo);
//						} catch (Exception e) {
//							e.printStackTrace();
//						}
//					}
//					throw new BussinessException("3091", "已经领取过或正在领取中，不能再次领取会员");
//				}

				//判断领取数量
				if(activityInfo.getGivecount() < vipList.size()+flaggivecount){
					throw new BussinessException("3090", "已经领取过或正在领取中，不能再次领取会员");
				}

				Thread thread = new VipGroupThread(activityInfo,vipList,this,vipConfigService);
				thread.start();
			}else{
				throw new BussinessException("3090", "已经领取过或正在领取中，不能再次领取会员");
			}
		}

		System.out.println("VipInfo="+JSON.toJSONString(activityInfo));
	}

	/**
	 * 复合产品-会员领取情况
	 *
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void queryGroupVip(VipInfo activityInfo) throws BussinessException {
		//查询活动关系表
		List<ActivityBean> list = activeGroupDao.getActivityInfos(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(list == null || list.size()<=0) {
			OrderInfo orderInfo =preOrdeDao.getOrder(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
			if(orderInfo != null) {
				throw new BussinessException("9088", "预定购用户不能赠送会员");
			}

			//查询本地是否有订购关系
			int i=activeGroupDao.queryOrder(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
			System.out.println("order relation----"+i+","+activityInfo.getUserid()+","+activityInfo.getCpid()+","+activityInfo.getSpid());
			if(i>0){
				throw new BussinessException("3092", "用户有订购，但无会员领取权益");
			}

			/**
			 * 没有查询到活动关系，去查询vac
			 *
			 */
//			JSONObject json = VacQueryUtil.queryVAC(activityInfo);
//			if(json==null){
//				throw new BussinessException("3093", "用户无订购不能赠送会员，查询VAC失败，请求稍后再试");
//			}else{
//				System.out.println(json.toJSONString());
//				String code = json.getString("resultcode");
//				if("0".equals(code)){
//					throw new BussinessException("3094", "用户无订购不能赠送会员，查询VAC有订购，请求稍后再试");
//				}else if("-1".equals(code)){
//					throw new BussinessException("3095", "用户无订购不能赠送会员，查询VAC无订购，不能领取会员");
//				}else{
//					throw new BussinessException("3093", "用户无订购不能赠送会员，查询VAC失败，请求稍后再试");
//				}
//			}

			//20220629 can.he 屏蔽

			//20230203 ou 去掉vac改为查cap
			/*JSONObject json = VacQueryUtil.queryVAC(activityInfo);
			if(json==null){
				throw new BussinessException("3093", "用户无订购不能赠送会员，请求稍后再试");
			}else{
				System.out.println(json.toJSONString());
				String code = json.getString("resultcode");
				if("0".equals(code)){
					throw new BussinessException("3094", "用户无订购不能赠送会员，请求稍后再试");
				}else if("-1".equals(code)){
					throw new BussinessException("3095", "用户无订购不能赠送会员，不能领取会员");
				}else{
					throw new BussinessException("3093", "用户无订购不能赠送会员，请求稍后再试");
				}
			}*/

			JSONObject json = VacQueryUtil.queryCAP(activityInfo,dictionaryDao);
			System.out.println("queryCAP: "+ json );
			if(json==null){
				throw new BussinessException("3093", "用户无订购不能赠送会员，查询CAP失败，请求稍后再试");
			}else{
				String code = json.getString("resultCode");
				if("0".equals(code)){
					throw new BussinessException("3094", "用户无订购不能赠送会员，查询CAP有订购，请求稍后再试");
				}else if("-1".equals(code)){
					throw new BussinessException("3095", "用户无订购不能赠送会员，查询CAP无订购，不能领取会员");
				}else{
					throw new BussinessException("3093", "用户无订购不能赠送会员，查询CAP失败，请求稍后再试");
				}
			}
		}





		//判断省份
		String province = list.get(0).getProvince();
		for(ActivityBean bean :list){
			if(province == null){
				throw new BussinessException("5029", "用户省份信息为空");
			}else{
				if(!province.equals(bean.getProvince())){
					throw new BussinessException("5030", "用户省份信息不一致");
				}
			}
		}

		//会员领取省份限制
		String cangive = giveVipProvinceCrontrolDao.queryProvinceCrontrol(activityInfo.getCpid(), activityInfo.getSpid(), province, activityInfo.getActivetype());
		if(!"0".equals(cangive)) {
			throw new BussinessException("9098", "该省份会员被限制领取");
		}

		//黑名单
		VipWhiteBean vipWhiteBean = vipWhiteDao.getCount("0", activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		if(vipWhiteBean != null && "1".equals(vipWhiteBean.getType())){
			throw new BussinessException("9089", "黑名单用户禁止领取");
		}

		//三户信息-会员赠送限制
		// 20230203优化接口去掉该查询
//		ecbVipService.queryEcb(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(),province, 0, activityInfo.getActivetype());

		//调cap综合用户查询和卡部信控查询并记录
		capKbxkQryService.capKbxkQry(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());

		List<Vip> vips = new ArrayList<Vip>();
		boolean flagvips = false;//是否有会员可以领取
		boolean flaggive = false;//是否已经领取过会员
		for(ActivityBean bean :list ){
			Vip vip = new Vip();
			vip.setCpid(bean.getCpid());
			vip.setSpid(bean.getSpid());
			vip.setSpname(bean.getSpname());
			boolean flagvip = false;//是否已经成功领取会员
			try {
				//订购状态判断
				if("1".equals(bean.getOrderstatus())) {
					if(NumberUtils.toLong(bean.getEndtime())<NumberUtils.toLong(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss"))){
						throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
					}
				}else if (!"0".equals(bean.getOrderstatus())){
					throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
				}

				//会员领取省份限制
				String cangiveSun = giveVipProvinceCrontrolDao.queryProvinceCrontrol(bean.getCpid(), bean.getSpid(), province, activityInfo.getActivetype());
				if(!"0".equals(cangiveSun)) {
					throw new BussinessException("9098", "该省份会员被限制领取");
				}

				//黑名单
				VipWhiteBean vipWhiteBeanSun = vipWhiteDao.getCount("0", bean.getUserid(),bean.getCpid(),bean.getSpid());
				if(vipWhiteBeanSun != null && "1".equals(vipWhiteBeanSun.getType())){
					throw new BussinessException("9089", "黑名单用户禁止领取");
				}

				//领取状态判断
				if("4".equals(bean.getVipstatus())) {
					throw new BussinessException("9090", "该活动已暂停赠送会员");
				}else if("3".equals(bean.getVipstatus())) {
					flaggive = true;
					throw new BussinessException("9091", "会员赠送中");
				}else if("990099".equals(bean.getVipstatus())) {
					flaggive = true;
					throw new BussinessException("9091", "会员赠送中");
				}else if("1".equals(bean.getVipstatus())) {
					flaggive = true;
					throw new BussinessException("9086", "会员领取失败");
				}else if("0".equals(bean.getVipstatus())) {
					flaggive = true;
					flagvip = true;
					throw new BussinessException("9087", "会员已领取");
				}else if(!"2".equals(bean.getVipstatus())) {
					throw new BussinessException("9090", "会员领取状态异常");
				}
			} catch (Exception e) {
				if(e instanceof BussinessException) {
					vip.setResultcode(((BussinessException) e).getCode());
					vip.setErrorinfo(e.getMessage());

					//已经领取，则查询当前领取信息
					if(flagvip){
						try {
							//查询领取日志
							UserMsgBean userMsgBean = userMsgDao.getMsg(bean.getUserid(), bean.getCpid(),bean.getSpid(), activityInfo.getActivetype());
							if(userMsgBean != null) {
								vip.setUsertype(userMsgBean.getMsg2());
								vip.setUserval(userMsgBean.getMsg1());
//								vip.setVipname(userMsgBean.getMsg3());
							}
						} catch (Exception e2) {}
					}
				}else {
					vip.setResultcode("系统忙，请稍后再试");
					vip.setErrorinfo("8000");
				}
			}

			//只要有一个子产品可以领取，就返回正常
			if("0".equals(vip.getResultcode())){
				flagvips = true;
			}
			vips.add(vip);
		}

		if(!flagvips){
			activityInfo.setCode("3090");
			activityInfo.setInfo("已经领取过或正在领取中，不能再次领取会员");
		}else{
			if(flaggive){
				activityInfo.setCode("3091");
				activityInfo.setInfo("已经领取过或正在领取中，不能再次领取会员");
			}
		}
		activityInfo.setVips(vips);
	}


	public ActiveDao getActiveDao(){
		return activeDao;
	}
	public UserMsgDao getUserMsgDao(){
		return  userMsgDao;
	}


	/**
	 * 腾讯会员领取
	 *
	 * @return
	 */
	public GiveVipService getTencentService(){
		return giveVipService;
	}

	/**
	 * 优酷会员领取
	 *
	 * @return
	 */
	public YoukuVipService getYoukuService(){
		return youkuVipService;
	}

	/**
	 * 爱奇异会员月领取
	 *
	 * @return
	 */
	public AiqiyiGiveMonthVipService  getAiqyMonthService(){
		return aiqiyiGiveMonthVipService;
	}

	/**
	 * cctv会员领取
	 *
	 * @return
	 */
	public CtvGiveVipService  getCctvMonthService(){
		return ctvGiveVipService;
	}


	/**
	 * 乐视会员领取
	 * @return
	 */
	public LeshiGiveVipService  getLeshiService(){
		return leshiGiveVipService;
	}

	/**
	 * 乐视会员领取(新接口)
	 * @return
	 */
	public LeshiNewGiveVipService  getLeshiNewService(){
		return leshiNewGiveVipService;
	}

	/**
	 * 芒果会员领取
	 *
	 * @return
	 */
	public MangtvGiveVipService  getMangtv16Service(){
		return mangtvGiveVipService;
	}

	/**
	 * 芒果会员领取
	 *
	 * @return
	 */
	public GetVipService  getVipService(){
		return getVipService;
	}

}
