package com.chinacreator.video.service;


import java.net.URLEncoder;

import org.apache.commons.lang.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.chinacreator.video.bean.ActivityConfigBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActiveMango3JSDao;
import com.chinacreator.video.mapper.ActivityConfigDao;
import com.chinacreator.video.mapper.SysDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.SHAUtil;
import com.chinacreator.video.util.URLUtil;
import com.chinacreator.video.util.UsermobUtil;
import com.chinacreator.video.util.VipChangeUtil;

@Component
public class VipMango3JSService {
	
	private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");
	
	//只能领取此活动会员
	private static String  activetype = "303";
	
	private static String vipid="21594";
	private static String vippwd="m9fhbs3g";
	private static String orderid_p = "mango3js_";
	
	@Autowired
	private RedisService redisService;
	
	@Autowired
	private ComonService comonService;

	@Autowired
	private ActiveMango3JSDao activeDao;
	
	@Autowired
	private ActivityConfigDao activityConfigDao;
	
	@Autowired
	private SysDao sysdao;

	@Autowired
	VipChangeUtil vipChangeUtil;
	/**
	 * 并发请求
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		try {
			String keys = "joinvip_" + vipInfo.getActivetype() + "_" + vipInfo.getUserid();
			long count = redisService.incr(keys);
			if (count > 1) {
				throw new BussinessException("8001", "并发请求，请稍后再试");
			}
			
			vipInfo.setVippool(keys);
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw e;
			}else{
				ErrorLOGGER.error("", e);  
			}
		}
	}
	
	
	/**
	 * 活动相关信息校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkMango3(VipInfo vipInfo) throws BussinessException {
		if(!activetype.equals(vipInfo.getActivetype())){
			throw new BussinessException("9083", "活动无效");
		}
	}
	
	public void checkActivity(VipInfo vipInfo) throws BussinessException {
		ActivityConfigBean activityConfigBean = activityConfigDao.getConfig(vipInfo.getActivetype());
		if(activityConfigBean == null || !"0".equals(activityConfigBean.getStatus()) || 
				NumberUtils.toLong(sysdao.getSysTime())>NumberUtils.toLong(activityConfigBean.getEndtime()) || 
				NumberUtils.toLong(sysdao.getSysTime())<NumberUtils.toLong(activityConfigBean.getBegintime()) ) {
			throw new BussinessException("9083", "活动无效");
		}
		
		String [] Channels = activityConfigBean.getChannel().split(",");
		if(Channels.length!=2) {
			throw new BussinessException("9084", "活动配置异常");
		}
		
		if(vipInfo.getChannel() == null || "".equals(vipInfo.getChannel().trim())) {
			throw new BussinessException("9056", "channel不能为空");
		}
		if(!vipInfo.getChannel().equals(Channels[0])) {
			throw new BussinessException("9057", "channel校验失败");
		}
		
		vipInfo.setCpid(activityConfigBean.getCpid());
		vipInfo.setSpid(activityConfigBean.getSpid());
		vipInfo.setPwd(Channels[1]);
	}
	
	
	public void checkVipSign(VipInfo vipInfo) throws BussinessException {
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		String pwd = vipInfo.getPwd();
		String localSign = MD5.MD5Encode(vipInfo.getActivetype()+ vipInfo.getUserid()+ vipInfo.getChannel() + pwd + vipInfo.getTimestamp());
		if (!localSign.equals(vipInfo.getSignature())) {
			throw new BussinessException("9055", "signature校验失败");
		}
	}
	
	public void checkVipMob(VipInfo vipInfo) throws BussinessException {
		String userid = vipInfo.getUserid();
		if(StringUtils.isEmpty(userid)) {
			throw new BussinessException("9058", "userid不能为空");
		}
		
		String pwd = vipInfo.getPwd();
		try {
			userid = DesUtil.decode(userid, pwd);
			if(!UsermobUtil.isValid(userid)) {
				throw new BussinessException("9059", "手机号码异常");
			}
		}catch(Exception e) {
			throw new BussinessException("9060", "userid解密错误");
		}
		
		vipInfo.setUserid(userid);
	}
	
	/**
	 * 校验是否已经领取
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkJoinVip(VipInfo vipInfo) throws BussinessException {
		
		int joincount = activeDao.getJoinRecord(vipInfo.getUserid());
		if(joincount > 0) {
			throw new BussinessException("9085", "已经参与过本次活动，不能重复参与");
		}
		
	}
	
	
	
	/**
	 * 领取校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVip(VipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);//并发请求
		this.checkMango3(vipInfo);//判断活动代码
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkActivity(vipInfo);
		this.checkVipSign(vipInfo);
		this.checkVipMob(vipInfo);
		
		this.checkJoinVip(vipInfo);
	}
	
	/**
	 * 会员领取
	 * 
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void giveVip(VipInfo activityInfo) throws BussinessException {
		String orderid = orderid_p + activityInfo.getUserid();
		
		/**
		 *  保存会员领取
		 */
		int i=activeDao.insert(activityInfo.getUserid(), activityInfo.getActivetype(), vipid, "1", orderid);
		if(i>0) {
			
			try {
				
				//领取会员
				vipChangeUtil.getVip(vipid, activityInfo.getUserid(), activityInfo.getActivetype(), vippwd, "4", activityInfo.getUserid(), orderid, null ,activityInfo.getType()+"");
			
			} catch (BussinessException e) {
				activeDao.deteleJoining(activityInfo.getUserid());
				
				throw new BussinessException("9086", "会员充值失败，请稍后再试");
			}
			
			activeDao.joinSucceed(activityInfo.getUserid(), "0");
			
			/**
			 * 发送短信
			 */
			try {
				int count15 = activeDao.queryOrderRelation15(activityInfo.getUserid());
				if(count15<=0) {
					String smsid="5003102";
					String pwd= "soews92d";
					String url ="http://111.206.133.54/smsbusi/sms/send";
					String content = "【中国联通】"+activityInfo.getUserid()+"，您已成功领取3天芒果TV VIP会员。现在订购wo+视频芒果TV定向流量包月，还可免流看视频，快来点击订购吧  https://v.wo.cn/mg15hd20";
					this.sendSMS(url, smsid, pwd, activityInfo.getUserid(), content);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			
		}else {
			throw new BussinessException("8707", "会员领取失败，请稍后再试");
		}
		
	}
	
	
	public void removeRedis(VipInfo vipInfo) {
		try {
			redisService.removeKey(vipInfo.getVippool());
		} catch (Exception e) {}
	}
	
	
	/**
	 * 发送短信
	 * 
	 * @param usermob
	 * @param content
	 * @return
	 */
	private String sendSMS (String dealurl,String smsid,String key, String usermob, String content){
		try {
			String timestamp = System.currentTimeMillis()/1000+"";
			String userid =  DesUtil.encode(usermob,  key);
			String sign = SHAUtil.shaEncode(smsid+userid+timestamp+content+key).toLowerCase();
			
			userid = URLEncoder.encode(userid,"utf-8");
			content = URLEncoder.encode(content,"utf-8");
			String url = "?userid="+userid+"&smsid="+smsid+"&timestamp="+timestamp+"&sign="+sign+"&content="+content;
			String resp = URLUtil.get(dealurl+url);
			
			System.out.println(url+","+resp);
			return resp;
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
}
