package com.chinacreator.video.service;


import java.text.SimpleDateFormat;
import java.util.*;

import com.chinacreator.video.mapper.*;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.time.DateFormatUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.ActivityBean;
import com.chinacreator.video.bean.ActivityConfigBean;
import com.chinacreator.video.bean.ActivityVipLogBean;
import com.chinacreator.video.bean.CpSpBean;
import com.chinacreator.video.bean.OrderInfo;
import com.chinacreator.video.bean.UserMsgBean;
import com.chinacreator.video.bean.Vip;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.bean.VipWhiteBean;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.service.handle.GetVipService;
import com.chinacreator.video.thread.VacQueryUtil;
import com.chinacreator.video.thread.VipMultiThread;
import com.chinacreator.video.thread.VipMultiUtil;
import com.chinacreator.video.util.MD5;

@Component
public class VipMultiService {

	private static final Logger log = LoggerFactory.getLogger("VipMultiService");

	@Autowired
	private ActivityConfigDao activityConfigDao;

	@Autowired
	private RedisService redisService;

	@Autowired
	private CpSpDao CpSpDao;

	@Autowired
	private ComonService comonService;

	@Autowired
	private MobService mobService;

	@Autowired
	private ActivityConfigService activityService;

	@Autowired
	private GiveVipService giveVipService;

	@Autowired
	private VipControlMultiService vipControlService;

	@Autowired
	private ActiveDao activeDao;

	@Autowired
	private ActiveGroupDao activeGroupDao;

	@Autowired
	private ActiveVipLogDao activeLogDao;

	@Autowired
	private UserMsgDao userMsgDao;

	@Autowired
	private PreOrderDao preOrdeDao;

//	@Autowired
//	private VipDao vipdao;

	@Autowired
	private VipWhiteDao vipWhiteDao;

	@Autowired
	private CtvGiveVipService ctvGiveVipService;

	@Autowired
	private AiqiyiGiveVipService aiqiyiGiveVipService;

	@Autowired
	private GiveVipProvinceCrontrolDao giveVipProvinceCrontrolDao;

	@Autowired
	private AiqiyiGiveMonthVipService aiqiyiGiveMonthVipService;

	@Autowired
	private LeshiGiveVipService leshiGiveVipService;

	@Autowired
	private LeshiNewGiveVipService leshiNewGiveVipService;

	@Autowired
	private MangtvGiveVipService mangtvGiveVipService;

	@Autowired
	private YoukuVipService youkuVipService;

	@Autowired
	private EcbVipService ecbVipService;

	@Autowired
	private ActivityVipConfigService vipConfigService;

	@Autowired
	private GetVipService getVipService;

	@Autowired
	private CapKbxkQryService capKbxkQryService;

	@Autowired
	private VipJoinSpidDao vipJoinSpidDao;

	@Autowired
	private OrderDao orderDao;

	@Autowired
	private SysDao sysDao;

	@Autowired
	private ComplementSendDao complementSendDao;

	@Autowired
	private DictionaryDao dictionaryDao;

	/**
	 * 复合产品 - 并发控制
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		long count = redisService.incr("joinvip_" + vipInfo.getActivetype() + "_"+ vipInfo.getCpid() + "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid());
		if (count > 1) {
			throw new BussinessException("8001", "并发请求，请稍后再试");
		}
	}

	/**
	 * 复合产品 - 并发控制
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedisQuery(VipInfo vipInfo) throws BussinessException {
		long count = redisService.incr("joinvip_query_" + vipInfo.getActivetype() + "_"+ vipInfo.getCpid() + "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid());
		if (count > 1) {
			throw new BussinessException("8001", "并发请求，请稍后再试");
		}
	}

	/**
	 * 复合产品 - 并发控制
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedisReset(VipInfo vipInfo) throws BussinessException {
		long count = redisService.incr("joinvip_reset_" + vipInfo.getActivetype() + "_"+ vipInfo.getCpid() + "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid());
		if (count > 1) {
			throw new BussinessException("8001", "并发请求，请稍后再试");
		}
	}

	/**
	 * 复合产品-Sign校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkMultiVipSign(VipInfo vipInfo) throws BussinessException {
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(vipInfo.getCpid(), vipInfo.getSpid());
		String pwd = cpSpBean.getPwd();
		String data = vipInfo.getActivetype()+vipInfo.getUserid() +vipInfo.getGivecpid() + vipInfo.getTimestamp()+ pwd;
		String localSign = MD5.MD5Encode(data);
//		System.out.println(sign);
//		System.out.println(localSign);
		if (!localSign.equals(sign)) {
			throw new BussinessException("9054", "signature校验失败");
		}

		vipInfo.setSptype(cpSpBean.getType());
		vipInfo.setPwd(cpSpBean.getPwd());
		vipInfo.setMobtype(cpSpBean.getMobtype());

//		System.out.println("pwd ----"+vipInfo.getPwd());
	}
	/**
	 * 复合产品-Sign校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkMultiVipSignQuery(VipInfo vipInfo) throws BussinessException {
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(vipInfo.getCpid(), vipInfo.getSpid());
		String pwd = cpSpBean.getPwd();
		String data = vipInfo.getActivetype()+vipInfo.getUserid() + vipInfo.getTimestamp()+ pwd;
		String localSign = MD5.MD5Encode(data);
//		System.out.println(sign);
//		System.out.println(localSign);
		if (!localSign.equals(sign)) {
			throw new BussinessException("9054", "signature校验失败");
		}

		vipInfo.setSptype(cpSpBean.getType());
		vipInfo.setPwd(cpSpBean.getPwd());
		vipInfo.setMobtype(cpSpBean.getMobtype());

//		System.out.println("pwd ----"+vipInfo.getPwd());


		//查询可以领取会员的个数
		try {
			ActivityConfigBean config = activityConfigDao.getConfig(vipInfo.getActivetype());
			vipInfo.setGivecount(Integer.parseInt(config.getGivecount()));
		} catch (Exception e) {
			vipInfo.setGivecount(0);
		}
	}


	/**
	 * 复合产品-会员领取数量校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVipGiveCount(VipInfo vipInfo) throws BussinessException {
		//查询可以领取会员的个数
		try {
			ActivityConfigBean config = activityConfigDao.getConfig(vipInfo.getActivetype());
			vipInfo.setGivecount(Integer.parseInt(config.getGivecount()));
		} catch (Exception e) {
			vipInfo.setGivecount(0);
		}

		//选择的领取会员验证
		if(vipInfo.getGivecpid()==null || "".equals(vipInfo.getGivecpid().trim())){
			throw new BussinessException("8006", "选择领取的会员为空");
		}

		//解析要领取的会员
		try {
			List<String> cpids = new ArrayList<>();
			String [] arrs = vipInfo.getGivecpid().trim().split(",");
			for(String  arr:arrs){
				if(arr != null && !"".equals(arr.trim())){
					cpids.add(arr.trim());
				}
			}
			vipInfo.setCpids(cpids);
		} catch (Exception e) {}

		//选择领取的会员异常
		if(vipInfo.getCpids() ==null || vipInfo.getCpids().size()<1){
			throw new BussinessException("8006", "选择领取的会员为空");
		}else if(vipInfo.getCpids().size()>vipInfo.getGivecount()){
			throw new BussinessException("8007", "选择领取的会员，已经超过可以领取的数量");
		}
	}

	/**
	 * 复合产品-会员设置3号不
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVipGive3inMonth(VipInfo vipInfo) throws BussinessException {
		Calendar cal = Calendar.getInstance();
		int dd = cal.get(Calendar.DAY_OF_MONTH);
		int hh = cal.get(Calendar.HOUR_OF_DAY);
		if(dd==3 || (dd==2 && hh>=23)){
			throw new BussinessException("8095", "当前不能调整会员续送设置，请在3号以后再调整");
		}

	}


	/**
	 * 复合产品-领取校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkMultiVip(VipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		this.checkMultiVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);

		this.checkVipGiveCount(vipInfo);
	}

	/**
	 * 复合产品-查询校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkMultiVipQuery(VipInfo vipInfo) throws BussinessException {
//		this.checkRedisQuery(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		this.checkMultiVipSignQuery(vipInfo);
		mobService.checkVipMob(vipInfo);

//		this.checkVipGiveCount(vipInfo);
	}

	/**
	 * 复合产品-设置校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkMultiVipSet(VipInfo vipInfo) throws BussinessException {
		this.checkRedisReset(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		this.checkMultiVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);

		//2号11点以后和3号不能设置
		this.checkVipGive3inMonth(vipInfo);

		this.checkVipGiveCount(vipInfo);
	}


	/**
	 * 复合产品-多会员领取
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void giveMultiVip(VipInfo activityInfo) throws BussinessException {
		//多项选择会员领取并发控制，及领取数量控制
		vipControlService.canGiveVip(activityInfo);

		//查询活动关系表
		List<ActivityBean> list = activeGroupDao.getActivityInfos(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(list == null || list.size()<=0) {
			OrderInfo orderInfo =preOrdeDao.getOrder(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
			if(orderInfo != null) {
				throw new BussinessException("9088", "预定购用户不能赠送会员");
			}

			//查询本地是否有订购关系
			int i=activeGroupDao.queryOrder(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
			System.out.println("order relation----"+i+","+activityInfo.getUserid()+","+activityInfo.getCpid()+","+activityInfo.getSpid());
			if(i>0){
				throw new BussinessException("3092", "用户有订购，但无会员领取权益");
			}

			throw new BussinessException("9085", "用户无订购不能赠送会员");
		}

		//判断省份
		String province = list.get(0).getProvince();
		for(ActivityBean bean :list){
			if(province == null){
				throw new BussinessException("5029", "省份信息为空");
			}else{
				if(!province.equals(bean.getProvince())){
					throw new BussinessException("5030", "省份信息异常");
				}
			}
		}

		//会员领取省份限制
		String cangive = giveVipProvinceCrontrolDao.queryProvinceCrontrol(activityInfo.getCpid(), activityInfo.getSpid(), province, activityInfo.getActivetype());
		if(!cangive.equals("0")) {
			throw new BussinessException("9098", "该省份会员被限制领取");
		}

		//黑名单
		boolean whiteFlag = true;
		VipWhiteBean vipWhiteBean = vipWhiteDao.getCount("0", activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		if(vipWhiteBean != null && "1".equals(vipWhiteBean.getType())){
			throw new BussinessException("9089", "黑名单用户禁止领取");
		}
		//白名单，跳过领取
		if(vipWhiteBean != null && "0".equals(vipWhiteBean.getType())){
			whiteFlag = false;
		}

		//三户信息-会员赠送限制
		ecbVipService.queryEcb(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(),province, 0, activityInfo.getActivetype());

		//调cap综合用户查询和卡部信控查询并记录
		capKbxkQryService.capKbxkQry(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		//会员领取
		if(whiteFlag){
			//领取会员
			List<ActivityBean> vipList = new ArrayList<ActivityBean>();
			//需要失效的会员
			List<ActivityBean> cancelVipList = new ArrayList<ActivityBean>();

			boolean flaggive = false;//是否已经领取过会员
			int countgive=0;//已经领取过的数量
			for(ActivityBean bean :list ){
				try {
					//订购状态判断
					if("1".equals(bean.getOrderstatus())) {
						if(NumberUtils.toLong(bean.getEndtime())<NumberUtils.toLong(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss"))){
							throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
						}
					}else if (!"0".equals(bean.getOrderstatus())){
						throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
					}

					//领取状态判断
					if("4".equals(bean.getVipstatus())) {
//						throw new BussinessException("9090", "该活动已暂停赠送会员");
						vailVipJoinSpidConf(activityInfo,bean,"4l",countgive,false); //4l代表领取的状态
						flaggive = true;
					}else if("3".equals(bean.getVipstatus())) {
//						flaggive = true;
						++countgive;
						throw new BussinessException("9091", "会员赠送中");
					}else if("1".equals(bean.getVipstatus())) {
//						flaggive = true;
						++countgive;
						throw new BussinessException("9086", "会员领取失败");
					}else if("0".equals(bean.getVipstatus())) {
//						flaggive = true;
//						++countgive;
//						throw new BussinessException("9087", "会员已领取");
						vailVipJoinSpidConf(activityInfo,bean,"0",countgive,false);
						flaggive = true;
					}else if(!"2".equals(bean.getVipstatus())) {
						throw new BussinessException("9090", "会员领取状态异常");
					}

					//会员领取省份限制
					String cangiveSun = giveVipProvinceCrontrolDao.queryProvinceCrontrol(bean.getCpid(), bean.getSpid(), province, activityInfo.getActivetype());
					if(!cangiveSun.equals("0")) {
						throw new BussinessException("9098", "该省份会员被限制领取");
					}

					//黑名单
					boolean whiteFlagSun = true;
					VipWhiteBean vipWhiteBeanSun = vipWhiteDao.getCount("0", bean.getUserid(),bean.getCpid(),bean.getSpid());
					if(vipWhiteBeanSun != null && "1".equals(vipWhiteBeanSun.getType())){
						throw new BussinessException("9089", "黑名单用户禁止领取");
					}
					//白名单，跳过领取
					if(vipWhiteBeanSun != null && "0".equals(vipWhiteBeanSun.getType())){
						whiteFlagSun = false;
					}

					//子产品会员领取
					if(whiteFlagSun){
						bean.setFhcpid(activityInfo.getCpid());
						bean.setFhspid(activityInfo.getSpid());
						//只有选择的会员会进行领取
						if(activityInfo.isSelectCpid(bean.getCpid())){
							//修改会员领取状态为:3(领取中)
							if(flaggive){
								bean.setOldvipstatus(bean.getVipstatus());
							}else {
								bean.setOldvipstatus("2");
							}
							bean.setNewvipstatus("3");
							int i =  activeGroupDao.updateVip(bean);
							if(i>0){
								vipList.add(bean);
							}
						}else{
							cancelVipList.add(bean);
						}
					}
					System.out.println("activityBean="+JSON.toJSONString(bean));
				} catch (Exception e) {
					if(e instanceof BussinessException) {
						bean.setVipresultcode(((BussinessException) e).getCode());
						bean.setVipresultinfo(e.getMessage());
					}else {
						bean.setVipresultcode("系统忙，请稍后再试");
						bean.setVipresultinfo("8000");
					}
				}
			}

			//已经领取过的 + 准备领取 > 能领取的数量，则异常
			if(countgive + vipList.size() > activityInfo.getGivecount()){
				throw new BussinessException("8007", "选择领取的会员，已经超过可以领取的数量");
			}

			//领取会员
			if(vipList !=null && vipList.size()>0){
				try {
					//设置所有会员不能续送
					activeLogDao.updateNotContinueVip(activityInfo);
				} catch (Exception e) {}

				//会员领取
				VipMultiUtil thread = new VipMultiUtil(activityInfo,vipList,this,vipConfigService,cancelVipList);
				List<Vip> vips = thread.run();
				activityInfo.setVips(vips);
			}else{
				throw new BussinessException("3090", "已经领取过或正在领取中，不能再次领取会员");
			}
		}

		System.out.println("VipInfo="+JSON.toJSONString(activityInfo));
	}


	/**
	 * 当活动表的会员领取状态为已领取时，判断是否当月还能再手动领取会员
	 * @return
	 * @throws Exception
	 */
	public void vailVipJoinSpidConf(VipInfo activityInfo, ActivityBean activityBean,String status,Integer countgive,boolean flagvip) throws BussinessException {
		try {
			boolean contains = false;
			String spid = activityInfo.getSpid();
			//获取手动赠送配置表的信息
			List<HashMap> dataList = vipJoinSpidDao.getVipJoinSpidConf(spid);
			if(dataList != null && dataList.size()>0){
				//获取活动表的订购信息
				OrderInfo orderInfo = new OrderInfo();
				orderInfo.setUserid(activityInfo.getUserid());
				orderInfo.setCpid(activityInfo.getCpid());
				orderInfo.setSpid(activityInfo.getSpid());
				//查询订购关系表获取订购信息
				OrderInfo order = orderDao.getOrder(orderInfo);
				if(order == null){
					throw new BussinessException("9085", "用户无订购不能赠送会员");
				}
				//获取子渠道
				String orderchannel2 = order.getOrderchannel2();
				//获取主渠道
				String orderchannel = order.getOrderchannel();
				log.info("手动领取会员配置："+dataList+"，orderchannel2=>"+orderchannel2+"，orderchannel=>"+orderchannel+"，userid=>"+activityInfo.getUserid()+"，spid=>"+activityInfo.getSpid());
				//子渠道标识或主渠道标识不为空
				if(!StringUtils.isEmpty(orderchannel2) || !StringUtils.isEmpty(orderchannel)){
					//渠道是否有配置
					for(HashMap hm : dataList){
						if("0".equals(hm.get("HASSUBCHANNEL"))){//子渠道
							//配置渠道与子渠道相同，SPID相同，当前月份与已领取月份不相同
							if(!StringUtils.isEmpty(orderchannel2)
									&& ("*".equals(hm.get("CHANNEL")) || orderchannel2.equals(hm.get("CHANNEL")))
									&& spid.equals(hm.get("SPID"))
									&& this.valiConfDate(activityInfo, activityBean,hm)){
								contains = true;
								break;
							}
						}else{//主渠道
							//配置渠道与主渠道相同，SPID相同，当前月份与已领取月份不相同
							if(!StringUtils.isEmpty(orderchannel)
									&& ("*".equals(hm.get("CHANNEL")) || orderchannel.equals(hm.get("CHANNEL")))
									&& spid.equals(hm.get("SPID"))
									&& this.valiConfDate(activityInfo, activityBean,hm)){
								contains = true;
								break;
							}
						}

					}
				}else{//orderchannel和orderchannel2都为空
					//渠道是否有配置
					for(HashMap hm : dataList){
						//配置渠道与主渠道相同，SPID相同，当前月份与已领取月份不相同
						if("*".equals(hm.get("CHANNEL"))
								&& spid.equals(hm.get("SPID"))
								&& this.valiConfDate(activityInfo, activityBean,hm)){
							contains = true;
							break;
						}
					}
				}
			}

			if(!contains){
				if(status.equals("4l")){ //领取的4
					throw new BussinessException("9090", "该活动已暂停赠送会员");
				}else if(status.equals("4q")){  //查询的4
					throw new BussinessException("9090", "会员已取消领取");
				}else if(status.equals("0q")){  //查询的0
					++countgive;
					flagvip = true;
					throw new BussinessException("9090", "会员已取消领取");
				}else{
					++countgive;
					throw new BussinessException("9087", "会员已领取");
				}
			}
		} catch (Exception e) {
			if(e instanceof BussinessException) {
				throw e;
			}else {
				throw new BussinessException("8000","验证手动赠送会员出现异常，"+e.getMessage());
			}
		}
	}


	/**
	 * 验证业务是否在配置时间内
	 * @param confmap 手动领取会员配置
	 * @param viptime 已领取会员时间
	 * @return
	 */
	private boolean valiConfDate(VipInfo activityInfo, ActivityBean activityBean, HashMap confmap){
		boolean res = false;
		String starttime = null;
		String endtime = null;
		String systime = sysDao.getSysTime();
		String viptime = activityBean.getViptime();
		String userid = activityInfo.getUserid();
		String spid = activityInfo.getSpid();

		//判断当月是否已领取过会员（系统时间的月份与已领取时间的月份相同）
		if(!StringUtils.isEmpty(viptime) && systime.substring(0, 6).equals(viptime.substring(0,6))){
			log.info("userid=>"+userid+"，spid=>"+spid+"，当月已领取，systime=>"+systime+"，viptime=>"+viptime);
			return res;
		}

		//判断开始时间和结束时间是否合法
		if(confmap.get("STARTTIME") != null && confmap.get("STARTTIME").toString().length() == 14 ){
			starttime = confmap.get("STARTTIME").toString();
		}
		if(confmap.get("ENDTIME") != null && confmap.get("ENDTIME").toString().length() == 14 ){
			endtime = confmap.get("ENDTIME").toString();
		}

		//有时间配置
		if(!StringUtils.isEmpty(starttime) || !StringUtils.isEmpty(endtime)){
			//开始时间和结束时间都有配置
			if(!StringUtils.isEmpty(starttime) && !StringUtils.isEmpty(endtime)
					&& Long.parseLong(systime) >= Long.parseLong(starttime)
					&& Long.parseLong(systime) <= Long.parseLong(endtime)){
				res = true;
				//只配置了开始时间
			}else if(!StringUtils.isEmpty(starttime) && StringUtils.isEmpty(endtime)
					&& Long.parseLong(systime) >= Long.parseLong(starttime)){
				res = true;
				//只配置了结束时间
			}else if(StringUtils.isEmpty(starttime) && !StringUtils.isEmpty(endtime)
					&& Long.parseLong(systime) <= Long.parseLong(endtime)){
				res = true;
			}
		}else{ //时间无配置
			res = true;
		}

		log.info("userid=>"+userid+"，spid=>"+spid+"，手动领取时间验证结果=>"+res+"，systime=>"+systime+"，starttime=>"+starttime+"，endtime=>"+endtime+"，viptime=>"+viptime);

		return res;
	}


	/**
	 * 复合产品-会员领取情况
	 *
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void queryMultiVip(VipInfo activityInfo) throws BussinessException {
		//查询活动关系表
		List<ActivityBean> list = activeGroupDao.getActivityInfos(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(list == null || list.size()<=0) {
			OrderInfo orderInfo =preOrdeDao.getOrder(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
			if(orderInfo != null) {
				throw new BussinessException("9088", "预定购用户不能赠送会员");
			}

			//查询本地是否有订购关系
			int i=activeGroupDao.queryOrder(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
			System.out.println("order relation----"+i+","+activityInfo.getUserid()+","+activityInfo.getCpid()+","+activityInfo.getSpid());
			if(i>0){
				throw new BussinessException("3092", "用户有订购，但无会员领取权益");
			}

			/**
			 * 没有查询到活动关系，去查询vac
			 *
			 */
//			JSONObject json = VacQueryUtil.queryVAC(activityInfo);
//			if(json==null){
//				throw new BussinessException("3093", "用户无订购不能赠送会员，查询VAC失败，请求稍后再试");
//			}else{
//				System.out.println(json.toJSONString());
//				String code = json.getString("resultcode");
//				if("0".equals(code)){
//					throw new BussinessException("3094", "用户无订购不能赠送会员，查询VAC有订购，请求稍后再试");
//				}else if("-1".equals(code)){
//					throw new BussinessException("3095", "用户无订购不能赠送会员，查询VAC无订购，不能领取会员");
//				}else{
//					throw new BussinessException("3093", "用户无订购不能赠送会员，查询VAC失败，请求稍后再试");
//				}
//			}

			//20220629 can.he 屏蔽

			// 20230203 ou  去掉vac改为查cap
			/*JSONObject json = VacQueryUtil.queryVAC(activityInfo);
			if(json==null){
				throw new BussinessException("3093", "用户无订购不能赠送会员，请求稍后再试");
			}else{
				System.out.println(json.toJSONString());
				String code = json.getString("resultcode");
				if("0".equals(code)){
					throw new BussinessException("3094", "用户无订购不能赠送会员，请求稍后再试");
				}else if("-1".equals(code)){
					throw new BussinessException("3095", "用户无订购不能赠送会员，不能领取会员");
				}else{
					throw new BussinessException("3093", "用户无订购不能赠送会员，请求稍后再试");
				}
			}*/

			JSONObject json = VacQueryUtil.queryCAP(activityInfo,dictionaryDao);
			System.out.println("queryCAP: "+ json );
			if(json==null){
				throw new BussinessException("3093", "用户无订购不能赠送会员，查询CAP失败，请求稍后再试");
			}else{
				String code = json.getString("resultCode");
				if("0".equals(code)){
					throw new BussinessException("3094", "用户无订购不能赠送会员，查询CAP有订购，请求稍后再试");
				}else if("-1".equals(code)){
					throw new BussinessException("3095", "用户无订购不能赠送会员，查询CAP无订购，不能领取会员");
				}else{
					throw new BussinessException("3093", "用户无订购不能赠送会员，查询CAP失败，请求稍后再试");
				}
			}
		}

		//判断省份
		String province = list.get(0).getProvince();
		for(ActivityBean bean :list){
			if(province == null){
				throw new BussinessException("5029", "用户省份信息为空");
			}else{
				if(!province.equals(bean.getProvince())){
					throw new BussinessException("5030", "用户省份信息不一致");
				}
			}
		}

		//会员领取省份限制
		String cangive = giveVipProvinceCrontrolDao.queryProvinceCrontrol(activityInfo.getCpid(), activityInfo.getSpid(), province, activityInfo.getActivetype());
		if(!"0".equals(cangive)) {
			throw new BussinessException("9098", "该省份会员被限制领取");
		}

		//黑名单
		VipWhiteBean vipWhiteBean = vipWhiteDao.getCount("0", activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		if(vipWhiteBean != null && "1".equals(vipWhiteBean.getType())){
			throw new BussinessException("9089", "黑名单用户禁止领取");
		}

		//三户信息-会员赠送限制
		// 20230203优化接口去掉该查询
//		ecbVipService.queryEcb(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(),province, 0, activityInfo.getActivetype());

		//调cap综合用户查询和卡部信控查询并记录
		capKbxkQryService.capKbxkQry(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		List<Vip> vips = new ArrayList<Vip>();

		boolean canFlag = false;//是否有可以领取的会员
		int countgive = 0;//已经领取过的
//		int givecount = 0;//准备领取的
		for(ActivityBean bean :list ){
			Vip vip = new Vip();
			vip.setCpid(bean.getCpid());
			vip.setSpid(bean.getSpid());
			vip.setContinueflag("0".equals(bean.getVipconflag())?"0":"");
			boolean flagvip = false;//领取过会员

			try {
				//订购状态判断
				if("1".equals(bean.getOrderstatus())) {
					if(NumberUtils.toLong(bean.getEndtime())<NumberUtils.toLong(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss"))){
						throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
					}
				}else if (!"0".equals(bean.getOrderstatus())){
					throw new BussinessException("9084", "用户为无效订购状态不能赠送会员");
				}

				//会员领取省份限制
				String cangiveSun = giveVipProvinceCrontrolDao.queryProvinceCrontrol(bean.getCpid(), bean.getSpid(), province, activityInfo.getActivetype());
				if(!"0".equals(cangiveSun)) {
					throw new BussinessException("9098", "该省份会员被限制领取");
				}

				//黑名单
				VipWhiteBean vipWhiteBeanSun = vipWhiteDao.getCount("0", bean.getUserid(),bean.getCpid(),bean.getSpid());
				if(vipWhiteBeanSun != null && "1".equals(vipWhiteBeanSun.getType())){
					throw new BussinessException("9089", "黑名单用户禁止领取");
				}

				//领取状态判断
				if("4".equals(bean.getVipstatus())) {
//					throw new BussinessException("9090", "会员已取消领取");
					vailVipJoinSpidConf(activityInfo,bean,"4q",countgive,false);
				}else if("3".equals(bean.getVipstatus())) {
					++countgive;
					flagvip = true;
					throw new BussinessException("9091", "会员领取中");
				}else if("1".equals(bean.getVipstatus())) {
					++countgive;
					flagvip = true;
					throw new BussinessException("9086", "会员领取失败");
				}else if("0".equals(bean.getVipstatus())) {
//					++countgive;
//					flagvip = true;
//					throw new BussinessException("9087", "会员已领取");
					vailVipJoinSpidConf(activityInfo,bean,"0q",countgive,flagvip);
				}else if(!"2".equals(bean.getVipstatus())) {
					throw new BussinessException("9090", "会员领取状态异常");
				}

				canFlag = true;

//				//子产品可以领取
//				if(activityInfo.isSelectCpid(bean.getCpid())){
//					++givecount;
//
//					vip.setResultcode("0");
//					vip.setErrorinfo("");
//				}else{
//					vip.setResultcode("1");
//					vip.setErrorinfo("未选择领取");
//				}
			} catch (Exception e) {
				if(e instanceof BussinessException) {
					vip.setResultcode(((BussinessException) e).getCode());
					vip.setErrorinfo(e.getMessage());

					//已经领取过，则查询当前领取信息
					if(flagvip){
						try {
							//查询领取日志
							UserMsgBean userMsgBean = userMsgDao.getMsg(bean.getUserid(), bean.getCpid(),bean.getSpid(), activityInfo.getActivetype());
							if(userMsgBean != null) {
								vip.setUsertype(userMsgBean.getMsg2());
								vip.setUserval(userMsgBean.getMsg1());
							}
						} catch (Exception e2) {}
					}
				}else {
					vip.setResultcode("系统忙，请稍后再试");
					vip.setErrorinfo("8000");
				}
			}

			vips.add(vip);
		}

		activityInfo.setVips(vips);

//		//选择的会员是否可以领取
//		if(givecount <= 0){
//			activityInfo.setCode("8008");
//			activityInfo.setInfo("选择领取的会员，已经领取或不能领取");
//		}

		//领取过  >= 可领取数
		if(countgive >= activityInfo.getGivecount()){
			activityInfo.setCode("8007");
			activityInfo.setInfo("已经领取会员或没有会员可以领取");
		}

		if(!canFlag){
			activityInfo.setCode("8038");
			activityInfo.setInfo("无会员可以领取");
		}


	}


	/**
	 * 复合产品-会员领取设置
	 *
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void setMultiVip(VipInfo activityInfo) throws BussinessException {
		//会员是否已经完全领取成功
		boolean flag =  vipControlService.isGiveOver(activityInfo);
		if(!flag){
			throw new BussinessException("8010", "会员未全部领取成功，不能调整续送会员类型");
		}

		if(activityInfo.getCpids().size() != activityInfo.getGivecount()){
			throw new BussinessException("8007", "选择领取的会员，与可以领取的数量不一致");
		}

		//查询活动关系表
		List<ActivityBean> list = activeGroupDao.getActivityInfos(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(list == null || list.size()<=0) {
			throw new BussinessException("8011", "未查询到活动关系");
		}

		//设置为可以领取的会员列表
		List<ActivityBean> updateList = new ArrayList<>();
		for(ActivityBean bean :list ){

			//失效状态不能修改
			if(bean.getEndtime() != null && !"".equals(bean.getEndtime())){
				long endtime = 0;
				try {
					endtime = new SimpleDateFormat("yyyyMMddHHmmss").parse(bean.getEndtime()).getTime();
				} catch (Exception e) {
					e.printStackTrace();
				}
				if(endtime<System.currentTimeMillis()){
					throw new BussinessException("8028", "订购已经失效不能修改");
				}
			}

			try {
				if(activityInfo.isSelectCpid(bean.getCpid())){
					updateList.add(bean);
				}
			} catch (Exception e) {}
		}

		if(updateList.size()==0){
			throw new BussinessException("8012", "选择续送的会员，没有到对应会员权益");
		}

		if(updateList.size() > activityInfo.getGivecount()){
			throw new BussinessException("8013", "选择续送的会员，已经超过可以续送的数量");
		}

		if(updateList.size() != activityInfo.getGivecount()){
			throw new BussinessException("8014", "选择续送的会员，与活动可以续送的数量不一致");
		}




		//先设置所有会员不能续送
		activeLogDao.updateNotContinueVip(activityInfo);

		//再设置可以续送的会员
		String vipname="";
		for(ActivityBean bean :updateList ){
			try {
				//日志中会员的名称
				if("tencent".equals(bean.getCpid())){
					vipname=vipname+"，腾讯视频月会员";
				}else if("aiqy".equals(bean.getCpid())){
					vipname=vipname+"，爱奇艺视频月会员";
				}else if("youtu".equals(bean.getCpid())){
					vipname=vipname+"，优酷视频月会员";
				}

				//保存领取帐号
				if(!"0".equals(bean.getVipstatus()) && !"0".equals(bean.getVipconflag())){
					userMsgDao.insertMsg(getUserMsgBean(bean));
				}

				//设置为可以续送
				activeLogDao.updateCanContinueVip(bean);
			} catch (Exception e) {}
		}
		vipname= vipname+" 。";


		//保存修改日志
		try {
			ActivityVipLogBean  log = new ActivityVipLogBean();
			log.setUserid(activityInfo.getUserid());
			log.setActivetype(activityInfo.getActivetype());
			log.setCpid(activityInfo.getCpid());
			log.setSpid(activityInfo.getSpid());
			log.setGivecpid(activityInfo.getGivecpid());
			log.setLogtype("续送会员设置");
			log.setRemark("续送会员设置成功："+vipname.substring(1));
			activeLogDao.insertLog(log);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}


	public ActiveVipLogDao getActiveVipLogDao(){
		return activeLogDao;
	}
	public ActiveDao getActiveDao(){
		return activeDao;
	}
	public UserMsgDao getUserMsgDao(){
		return  userMsgDao;
	}

	public ComplementSendDao getComplementSendDao(){
		return complementSendDao;
	}

	/**
	 * 腾讯会员领取
	 *
	 * @return
	 */
	public GiveVipService getTencentService(){
		return giveVipService;
	}

	/**
	 * 优酷会员领取
	 *
	 * @return
	 */
	public YoukuVipService getYoukuService(){
		return youkuVipService;
	}

	/**
	 * 爱奇异会员月领取
	 *
	 * @return
	 */
	public AiqiyiGiveMonthVipService  getAiqyMonthService(){
		return aiqiyiGiveMonthVipService;
	}

	/**
	 * cctv会员领取
	 *
	 * @return
	 */
	public CtvGiveVipService  getCctvMonthService(){
		return ctvGiveVipService;
	}


	/**
	 * 乐视会员领取
	 * @return
	 */
	public LeshiGiveVipService  getLeshiService(){
		return leshiGiveVipService;
	}

	/**
	 * 乐视会员领取(新接口)
	 * @return
	 */
	public LeshiNewGiveVipService  getLeshiNewService(){
		return leshiNewGiveVipService;
	}

	/**
	 * 芒果会员领取
	 *
	 * @return
	 */
	public MangtvGiveVipService  getMangtv16Service(){
		return mangtvGiveVipService;
	}

	public GetVipService  getVipService(){
		return getVipService;
	}

	public UserMsgBean getUserMsgBean(ActivityBean activityInfo) {
		UserMsgBean userMsgBean = new UserMsgBean();
		userMsgBean.setActiveid(activityInfo.getActiveType());
		userMsgBean.setUserid(activityInfo.getUserid());
		userMsgBean.setCpid(activityInfo.getCpid());
		userMsgBean.setSpid(activityInfo.getSpid());
		userMsgBean.setMsg1(activityInfo.getUserid());
		userMsgBean.setMsg2("4");
		userMsgBean.setMsg3("");
		System.out.println("UserMsgBean="+JSON.toJSONString(activityInfo));
		return userMsgBean;
	}

}
