package com.chinacreator.video.service;


import java.text.SimpleDateFormat;
import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.chinacreator.video.bean.UserMsgBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.bean.VipWhiteBean;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActiveYouku7Dao;
import com.chinacreator.video.mapper.GiveVipProvinceCrontrolDao;
import com.chinacreator.video.mapper.UserMsgDao;
import com.chinacreator.video.mapper.VipWhiteDao;
import com.chinacreator.video.redis.RedisService;

@Component
public class VipYouku7Service {

	//只能领取此活动会员
	private static String  activetype = "17";
	//活动结束时间
	private static String activeend = "20190924235959";

	//活动结束时间
	private static int count = 3000;

	//redis计数
	private static String key = "joinvip_count_" + activetype + "_totals_20190924235959";

	private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");

	@Autowired
	private RedisService redisService;

	@Autowired
	private ComonService comonService;

	@Autowired
	private MobService mobService;

	@Autowired
	private ActivityConfigService activityService;

	@Autowired
	private ActiveYouku7Dao activeDao;

	@Autowired
	private UserMsgDao userMsgDao;

	@Autowired
	private VipWhiteDao vipWhiteDao;

	@Autowired
	private GiveVipProvinceCrontrolDao giveVipProvinceCrontrolDao;

	@Autowired
	private EcbVipService ecbVipService;

	@Autowired
	private Youku7VipService youku7VipService;

	@Autowired
	private CapKbxkQryService capKbxkQryService;

	public long getRedis(VipInfo vipInfo) throws BussinessException {
		try {
			String count = redisService.get(key);
			System.out.println(key+":"+count);
			return Long.parseLong(count);
		} catch (Exception e) {
			ErrorLOGGER.error("", e);
		}
		return 0;
	}


	public long addRedis(VipInfo vipInfo) throws BussinessException {
		try {
			long count = redisService.incrAdd(key,60L*60L*24L*100L);
			System.out.println(key+":"+count);
			return count;
		} catch (Exception e) {
			ErrorLOGGER.error("", e);
		}
		return 0;
	}

	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		try {
			long count = redisService.incr("joinvip_" + vipInfo.getActivetype() + "_" + vipInfo.getUserid());
			if (count > 1) {
				throw new BussinessException("8001", "并发请求，请稍后再试");
			}
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw e;
			}else{
				ErrorLOGGER.error("", e);
			}
		}
	}

	/**
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkYouku7(VipInfo vipInfo) throws BussinessException {

		if(!activetype.equals(vipInfo.getActivetype())){
			throw new BussinessException("9083", "该活动无效");
		}

		try {
			if(new SimpleDateFormat("yyyyMMddHHmmss").parse(activeend).getTime()<System.currentTimeMillis()){
				throw new BussinessException("9083", "该活动已经结束");
			}
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw (BussinessException)e;
			}else{
				ErrorLOGGER.error("", e);
			}
		}
	}


	/**
	 * 领取校验
	 *
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVip(VipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);
		this.checkYouku7(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		comonService.checkQueryVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);
	}

	public void checkVipQuery(VipInfo vipInfo) throws BussinessException {
		this.checkYouku7(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		comonService.checkQueryVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);
	}

	/**
	 * 会员领取
	 *
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void giveVip(VipInfo activityInfo) throws BussinessException {
		//是否已经领取
		int joincount = activeDao.getJoinVip(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(joincount > 0) {
			throw new BussinessException("6081", "亲爱的客官您已经领过礼品");
		}

		//是否有效订购
		int ordercount = activeDao.getOrderRelation(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
		if(ordercount<=0){
			throw new BussinessException("6082", "用户为无效订购状态");
		}

		//是否已经领取完
		int vipcount = activeDao.getActivetypeCount(activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(vipcount>= count){
			throw new BussinessException("6083", "亲爱的客官您来晚一步，礼品已经领光啦");
		}

		long vipcount2 = this.getRedis(activityInfo);
		if(vipcount2 >= count){
			throw new BussinessException("6083", "亲爱的客官您来晚一步，礼品已经领光啦");
		}

		//省份信息
		String province = activeDao.getOrderPrivince(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
		if(province==null || "".equals(province.trim())){
			throw new BussinessException("6084", "省份信息异常");
		}

		//会员赠送限制
		ecbVipService.queryEcb(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(),province, activityInfo.getType(), activityInfo.getActivetype());

		//调cap综合用户查询和卡部信控查询并记录
		capKbxkQryService.capKbxkQry(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		//省份限制
		String cangive = giveVipProvinceCrontrolDao.queryProvinceCrontrol(activityInfo.getCpid(), activityInfo.getSpid(), province, activityInfo.getActivetype());
		if(!"0".equals(cangive)) {
			throw new BussinessException("9098", "该省份会员被限制领取");
		}

		VipWhiteBean vipWhiteBean = vipWhiteDao.getCount("0", activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
		if(vipWhiteBean == null) {
			//会员领取
			if("youtu".equals(activityInfo.getCpid())){
				youku7VipService.giveVip(activityInfo);
			}else{
				throw new BussinessException("9088", "该cp暂未开放");
			}

		}else if("1".equals(vipWhiteBean.getType())) {
			throw new BussinessException("9089", "黑名单用户禁止领取");
		}
		System.out.println("VipInfo="+JSON.toJSONString(activityInfo));

		//保存领取日志
		String id = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()) + String.valueOf((int)((Math.random()*9+1)*1000));
		activeDao.insert(id, activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid(), activityInfo.getActivetype(), "0", province);

		userMsgDao.insertMsg(getUserMsgBean(activityInfo));

		//redis计数
		this.addRedis(activityInfo);
	}

	public UserMsgBean getUserMsgBean(VipInfo activityInfo) {
		UserMsgBean userMsgBean = new UserMsgBean();
		userMsgBean.setActiveid(activityInfo.getActivetype());
		userMsgBean.setCpid(activityInfo.getCpid());
		userMsgBean.setMsg1(activityInfo.getUserval());
		userMsgBean.setMsg2(activityInfo.getUsertype());
		userMsgBean.setMsg3(activityInfo.getVipname() == null ?"":activityInfo.getVipname());
		userMsgBean.setSpid(activityInfo.getSpid());
		userMsgBean.setUserid(activityInfo.getUserid());
		System.out.println("UserMsgBean="+JSON.toJSONString(activityInfo));
		return userMsgBean;
	}


	public void queryVip(VipInfo activityInfo) throws BussinessException {
		//是否已经领取
		int joincount = activeDao.getJoinVip(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(joincount > 0) {
			throw new BussinessException("6081", "亲爱的客官您已经领过礼品");
		}

		//是否有效订购
		int ordercount = activeDao.getOrderRelation(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
		if(ordercount<=0){
			throw new BussinessException("6082", "用户为无效订购状态");
		}

		//是否已经领取完
		int vipcount = activeDao.getActivetypeCount(activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
		if(vipcount >= count){
			throw new BussinessException("6083", "亲爱的客官您来晚一步，礼品已经领光啦");
		}

		long vipcount2 = this.getRedis(activityInfo);
		if(vipcount2 >= count){
			throw new BussinessException("6083", "亲爱的客官您来晚一步，礼品已经领光啦");
		}


	}

}
