package com.chinacreator.video.service;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.log.VipLog;
import com.chinacreator.video.mapper.SysDao;
import com.chinacreator.video.mapper.VipDao;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.URLUtil;

/**
 * 
 * PPTV会员领取
 * 
 * @author yanlong
 *
 */
@Service
public class WasuGiveVipService {
    private static final String VIPURL = "http://itf.upm.wasu.tv/pc-third-interface/cucc/orderSucc.do";       //prod

    private static final String ORIGIN = "CHINA_UNICOM_GROUP";
    private static final String PAY_CHANNEL = "3";
    private static final String USER_IP = "218.76.52.117";
    private static final String USER_MAC = "1A-FF-28-91-B2-20";
    private static final String PRICE = "20.0";
    private static final String KEY="llcuccsop1009slpyhgzgs";
    private static final String PRODUCTID_30 = "40414939485780000001721";
    
    private static final Logger log = LoggerFactory.getLogger("viplog");
    
    @Autowired
    private SysDao sysDao;
    @Autowired
    private VipDao vipDao;
    @Autowired
    VipControlService vipControlService;

    @Autowired
    ComplementSendService complementSendService;

    public void giveVip(VipInfo activityInfo) throws BussinessException {
        this.vipControlService.canGiveVip(activityInfo);

        VipLog vipLog = new VipLog();
        vipLog.setResultcode("0");
        vipLog.setErrorinfo("");
        vipLog.setActivityType(activityInfo.getActivetype());
        vipLog.setUserid(activityInfo.getUserid());
        vipLog.setType(activityInfo.getType());
        vipLog.setUsertype(activityInfo.getUsertype());
        vipLog.setUserval(activityInfo.getUserval());
        vipLog.setVipdays("31");
        
        //订单重试
        boolean flagCorp=false;
        String toCorpOrderid=null;
        if(activityInfo.getVippool()!=null && !"".equals(activityInfo.getVippool().trim())){
        	toCorpOrderid =  vipDao.getOrderId2(activityInfo.getUserid(), activityInfo.getActivetype(),activityInfo.getVippool());
        }else{
        	toCorpOrderid =  vipDao.getOrderId(activityInfo.getUserid(), activityInfo.getActivetype());
        }
        if(toCorpOrderid != null && !"".equals(toCorpOrderid)){
        	flagCorp=true;
        }

        try {
//            Map<String, String> param = new HashMap<String,String>();
            Map<String,String> param = new TreeMap<>();
            
			//重试订单
            String order_id = null;
            if(flagCorp){
            	order_id = toCorpOrderid;
            }else{
            	order_id = this.sysDao.getId();
            }

			String begintime = this.sysDao.getSysTime();
			String endtime = this.sysDao.getLastDay();

			vipLog.setOrderid(order_id);
			vipLog.setBegintime(begintime);
			vipLog.setEndtime(endtime);

			String phone  = activityInfo.getUserval();
			String resource_id = PRODUCTID_30;
            vipLog.setVipdays("30");
			
            //合作方直充接口参数
			param.put( "order_no", order_id);
            param.put( "order_name", ORIGIN);
            param.put( "success_time", begintime);
            param.put( "phone", phone);
            param.put( "user_ip", USER_IP);
            param.put( "user_mac", USER_MAC);
            param.put( "price", PRICE);
            param.put( "pay_channel", PAY_CHANNEL);
            param.put( "resource_id", resource_id);
            
            //签名MD5加密
            String paramUrl = URLUtil.mapToUrl(param) + "&key=" + KEY;
            log.debug("===before MD5 encode===\n" + paramUrl);
            String sign= MD5.MD5Encode(paramUrl);

            param.put("sign",sign);
            String urlParam = URLUtil.mapToUrl(param);
            String url = VIPURL + "?" + urlParam ;

			String result=null;
			try {
				vipLog.setOutput(JSON.toJSONString(param));
                result = URLUtil.post(url, "");
//              result = "{\"msg\":\"success\",\"code\":0,\"order_key\":\"40615395746410000007783ZT\"}";
				vipLog.setReturnput(result);
			} catch (Exception e) {
				vipLog.setResultcode("9070");
				vipLog.setErrorinfo("赠送会员超时或异常");
				throw new BussinessException("9070", "赠送会员超时或异常");
			}
			JSONObject obj = JSON.parseObject(result);
			
			//解析返回结果
			String msg = obj.getString("msg");
			String code = obj.getString("code");
			
			//订单是否成功
			boolean flagvip = false;
			if("0".equals(code)){
				flagvip=true;
			}
			
			try {
				//把原有的9070订单标记失败
				if(flagCorp && flagvip && vipDao.getCount9070(activityInfo.getUserid(), toCorpOrderid)>0){
					vipDao.updateOrderIdAll(activityInfo.getUserid(), toCorpOrderid, "1","订单重试表明：此订单失败");
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			vipLog.setResultcode(flagvip?"0":code);
			vipLog.setErrorinfo(msg);

			complementSendService.updateJoinAccount(activityInfo,vipLog.getResultcode(),msg,order_id);
			this.vipControlService.saveVipGive(activityInfo, vipLog.getResultcode());
			if (!flagvip) {
				throw new BussinessException("9071", "赠送会员失败(" + obj.getString("message") + ")");
			}
        }catch (Exception e){
            if (e instanceof BussinessException)
                throw (BussinessException) e;
            else
                throw new BussinessException("8000",e.getMessage());
        }finally{
            log.info(JSON.toJSONString(vipLog));
            saveLog(vipLog);
        }
    }

    public void saveLog(VipLog vipLog){
//    	vipLog.setResultcode("1");
    	
    	if(vipLog.getReturnput()==null){
			vipLog.setReturnput("");
		}
		if(vipLog.getResultcode()==null){
			vipLog.setResultcode("-0");
		}
		if(vipLog.getErrorinfo()==null){
			vipLog.setErrorinfo("");
		}
        this.vipDao.insert(vipLog);
    }

}
