package com.chinacreator.video.service.vip;


import java.util.List;

import org.apache.commons.lang.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.HxOrder;
import com.chinacreator.video.bean.HxProduct;
import com.chinacreator.video.bean.HxSubProduct;
import com.chinacreator.video.bean.HxVipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.HxOrderDao;
import com.chinacreator.video.mapper.HxOrderGroupDao;
import com.chinacreator.video.mapper.VipDao;
import com.chinacreator.video.service.ComonService;
import com.chinacreator.video.util.AESUtil;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.HxVipUtil;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.UsermobUtil;

@Component
public class HxOrderGroupService {
	
	private static final String MOB_KEY = "ac22273abb2f4960";
	
	@Autowired
	private HxOrderDao hxDao;
	
	@Autowired
	private HxOrderGroupDao hxGroupDao;
	
	@Autowired
	private VipDao vipDao;

	@Autowired
	private ComonService comonService;

	@Autowired
	private HxVipUtil hxVipUtil;
	
	/**
	 * Sign校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkSign(HxVipInfo vipInfo) throws BussinessException {
		if (StringUtils.isEmpty(vipInfo.getOrderid())) {
			throw new BussinessException("9054", "orderid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getUserid())) {
			throw new BussinessException("9054", "userid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getCpid())) {
			throw new BussinessException("9054", "cpid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getSpid())) {
			throw new BussinessException("9054", "spid不能为空");
		}
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		
		HxProduct goods = hxDao.getProdcutBean(vipInfo.getCpid(), vipInfo.getSpid());
		
		String pwd = goods.getPwd();
		String data = vipInfo.getOrderid() + vipInfo.getUserid() + vipInfo.getTimestamp() + pwd;
		String localSign = MD5.MD5Encode(data);
		if (!localSign.equals(sign)) {
			throw new BussinessException("9054", "signature校验失败");
		}
		
		vipInfo.setPwd(pwd);
	}
	
	/**
	 * userid校验与解密
	 * 
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void checkMob(HxVipInfo vipInfo) throws BussinessException {
		String userid = vipInfo.getUserid();
		try {
			userid = DesUtil.decode(userid, vipInfo.getPwd());
			if(userid.length() > 11) {
				userid = AESUtil.decrypt(userid, MOB_KEY);
			}
			if(!UsermobUtil.isValid(userid)) {
				throw new BussinessException("9055", "userid不正确");
			}
		}catch(Exception e) {
			throw new BussinessException("9056", "userid加密错误");
		}
		vipInfo.setUserid(userid);
	}
	
	
	
	/**
	 * 判断订单号
	 * 
	 * @param info
	 * @throws BussinessException
	 */
	public void checkOrderID(HxVipInfo info) throws BussinessException {
		try {
			HxOrder bean = hxDao.getOrderBean(info.getUserid(), info.getOrderid(), info.getCpid(), info.getSpid());
			if(bean == null ) {
				throw new BussinessException("9057", "此订单不存在");
			}
			
			info.setOrder(bean);
		} catch (Exception e) {
			e.printStackTrace();
			if (e instanceof BussinessException)
				throw (BussinessException) e;
			else
			    throw new BussinessException("9057","此订单查询异常。");
		}
	}
	
	/**
	 * 判断子产品信息
	 * 
	 * @param info
	 * @throws BussinessException
	 */
	public void checkOrderSub(HxVipInfo info) throws BussinessException {
		int subCount = 0;
		List<HxSubProduct> list = hxGroupDao.querySubSpid(info.getCpid(), info.getSpid());
		if(list == null || list.size()<1) {
			throw new BussinessException("9058", "子产品配置异常");
		}
		subCount = list.size();
		
		int count = hxGroupDao.countVipSub(info.getUserid(), info.getOrderid(), info.getCpid(), info.getSpid());
		if(count==0) {
			int i = hxGroupDao.insertSub(info.getUserid(), info.getOrderid(), info.getCpid(), info.getSpid());
			if(i != subCount) {
				throw new BussinessException("9058", "子产品订单入库异常");
			}
		}else if(subCount != count){
			throw new BussinessException("9058", "子产品订单记录异常");
		}
		
		info.setList(list);
	}

	/**
	 * 查询是否已经送了会员
	 * 
	 * @param info
	 * @throws BussinessException
	 */
	public void checkVipOrderID(HxVipInfo info) throws BussinessException {
		int num=0;
		List<HxSubProduct> list  = info.getList();
		for(HxSubProduct sub:list) {
			num = num + NumberUtils.toInt(vipDao.getRequestId(sub.getVipid(),info.getOrderid()));
		}
		
		if(num >= list.size()) {
			throw new BussinessException("0", "会员权益充值成功");
		}else if(num>0) {
			throw new BussinessException("-1", "会员权益充值部分成功");
		}else {
			int numFail=0;
			for(HxSubProduct sub:list) {
				numFail = numFail + NumberUtils.toInt(vipDao.getRequestIdFail(sub.getVipid(),info.getOrderid()));
			}
			if(numFail > 0) {
				throw new BussinessException("1", "会员权益充值失败");	
			}
		}
	}
	
	/**
	 * 请求校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVip(HxVipInfo vipInfo) throws BussinessException {
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkSign(vipInfo);
		this.checkMob(vipInfo);
		
		this.checkOrderID(vipInfo);
		this.checkOrderSub(vipInfo);
		this.checkVipOrderID(vipInfo);
	}
	
	
	/**
	 * 会员领取
	 * 
	 * @param info
	 * @throws BussinessException
	 */
	public void giveVip(HxVipInfo info) throws BussinessException {
		HxOrder bean = info.getOrder();
		//赠送中
		if(!"2".equals(bean.getVipstatus())){
			throw new BussinessException("9059", "此订单不在[会员领取中]状态");
		}
		
		//调用直充接口送会员
		int num = 0;
		List<HxSubProduct> list  = info.getList();
		for(HxSubProduct sub:list) {
			try {
				String remark = null;
				if("tencent".equals(sub.getSubcpid())){
					remark = "会员产品";
				}
				
				String result = hxVipUtil.vipid(sub.getVipid(), sub.getVipchannel(), sub.getVippwd(), info.getUsertype(), info.getUserval(), info.getOrderid(),remark);
				if(result == null || "".equals(result.trim())) {
					throw new BussinessException("9070","赠送会员超时或异常。");
				}
				
				JSONObject obj = JSON.parseObject(result);
				String resultcode = obj.getString("resultcode");
				String errorinfo = obj.getString("errorinfo");
				if(resultcode == null || "".equals(resultcode.trim()) || "8000".equals(resultcode.trim())) {
					throw new BussinessException("9070","赠送会员超时或异常。");
				}
				
				if("0".equals(resultcode) || ("10002".equals(resultcode) && "orderid不能重复使用".equals(errorinfo)) ){
					num = num+1;
					
					try {
						hxGroupDao.updateSub(info.getUserid(), info.getOrderid(), sub.getSubcpid(), sub.getSubspid());
					} catch (Exception e) {}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		if(num >= list.size()) {
			throw new BussinessException("0", "会员权益充值成功");
		}else if(num>0) {
			throw new BussinessException("-1", "会员权益充值部分成功");
		}else {
			
			int numSuccess = 0;
			for(HxSubProduct sub:list) {
				numSuccess = numSuccess + NumberUtils.toInt(vipDao.getRequestId(sub.getVipid(),info.getOrderid()));
			}
			if(numSuccess >= list.size()) {
				throw new BussinessException("0", "会员权益充值成功");
			}else if(numSuccess>0) {
				throw new BussinessException("-1", "会员权益充值部分成功");
			}else {
				int numFail=0;
				for(HxSubProduct sub:list) {
					numFail = numFail + NumberUtils.toInt(vipDao.getRequestIdFail(sub.getVipid(),info.getOrderid()));
				}
				if(numFail > 0) {
					throw new BussinessException("1", "会员权益充值失败");	
				}
			}
		}
	}
	
	
	
}
