package com.chinacreator.video.util;

import javax.crypto.Cipher;
import java.io.ByteArrayOutputStream;
import java.security.*;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.HashMap;
import java.util.Map;


public class RSAUtils {

    /** */
    /**
     * 加密算法RSA
     */
    public static final String KEY_ALGORITHM = "RSA";

    /** */
    /**
     * 签名算法
     */
    public static final String SIGNATURE_ALGORITHM = "MD5withRSA";

    /** */
    /**
     * 获取公钥的key
     */
    private static final String PUBLIC_KEY = "RSAPublicKey";

    /** */
    /**
     * 获取私钥的key
     */
    private static final String PRIVATE_KEY = "RSAPrivateKey";

    /** */
    /**
     * RSA最大加密明文大小
     */
    private static final int MAX_ENCRYPT_BLOCK = 117;

    /** */
    /**
     * RSA最大解密密文大小
     */
    private static final int MAX_DECRYPT_BLOCK = 128;

    public static Map<String, Object> genKeyPair() throws Exception {
        KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance(KEY_ALGORITHM);
        keyPairGen.initialize(1024);
        KeyPair keyPair = keyPairGen.generateKeyPair();
        RSAPublicKey publicKey = (RSAPublicKey) keyPair.getPublic();
        RSAPrivateKey privateKey = (RSAPrivateKey) keyPair.getPrivate();
        Map<String, Object> keyMap = new HashMap<String, Object>(2);
        keyMap.put(PUBLIC_KEY, publicKey);
        keyMap.put(PRIVATE_KEY, privateKey);
        return keyMap;
    }

    public static String sign(byte[] data, String privateKey) throws Exception {
        byte[] keyBytes = Base64.decodeBase64(privateKey);
        PKCS8EncodedKeySpec pkcs8KeySpec = new PKCS8EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        PrivateKey privateK = keyFactory.generatePrivate(pkcs8KeySpec);
        Signature signature = Signature.getInstance(SIGNATURE_ALGORITHM);
        signature.initSign(privateK);
        signature.update(data);
        return Base64.encodeBase64String(signature.sign());
    }

    public static boolean verify(byte[] data, String publicKey, String sign)
            throws Exception {
        byte[] keyBytes = Base64.decodeBase64(publicKey);
        X509EncodedKeySpec keySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        PublicKey publicK = keyFactory.generatePublic(keySpec);
        Signature signature = Signature.getInstance(SIGNATURE_ALGORITHM);
        signature.initVerify(publicK);
        signature.update(data);
        return signature.verify(Base64.decodeBase64(sign));
    }

    public static byte[] decryptByPrivateKey(byte[] encryptedData, String privateKey)
            throws Exception {
        byte[] keyBytes = Base64.decodeBase64(privateKey);
        PKCS8EncodedKeySpec pkcs8KeySpec = new PKCS8EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key privateK = keyFactory.generatePrivate(pkcs8KeySpec);
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.DECRYPT_MODE, privateK);
        int inputLen = encryptedData.length;
        byte[] result = new byte[(encryptedData.length / 128 + 1) * 117];
        int offSet = 0;
        byte[] cache;
        int i = 0;
        int len = 0;
        // 对数据分段解密
        while (inputLen - offSet > 0) {
            if (inputLen - offSet > MAX_DECRYPT_BLOCK) {
                cache = cipher.doFinal(encryptedData, offSet, MAX_DECRYPT_BLOCK);
            } else {
                cache = cipher.doFinal(encryptedData, offSet, inputLen - offSet);
            }
            i++;
            offSet = i * MAX_DECRYPT_BLOCK;
            for (byte b : cache) {
                result[len++] = b;
            }
        }
        return result;
    }

    public static byte[] decryptByPublicKey(byte[] encryptedData, String publicKey)
            throws Exception {
        byte[] keyBytes = Base64.decodeBase64(publicKey);
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key publicK = keyFactory.generatePublic(x509KeySpec);
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.DECRYPT_MODE, publicK);
        int inputLen = encryptedData.length;
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        int offSet = 0;
        byte[] cache;
        int i = 0;
        // 对数据分段解密
        while (inputLen - offSet > 0) {
            if (inputLen - offSet > MAX_DECRYPT_BLOCK) {
                cache = cipher.doFinal(encryptedData, offSet, MAX_DECRYPT_BLOCK);
            } else {
                cache = cipher.doFinal(encryptedData, offSet, inputLen - offSet);
            }
            out.write(cache, 0, cache.length);
            i++;
            offSet = i * MAX_DECRYPT_BLOCK;
        }
        byte[] decryptedData = out.toByteArray();
        out.close();
        return decryptedData;
    }

    public static byte[] encryptByPublicKey(byte[] data, String publicKey)
            throws Exception {
        byte[] keyBytes = Base64.decodeBase64(publicKey);
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key publicK = keyFactory.generatePublic(x509KeySpec);
        // 对数据加密
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.ENCRYPT_MODE, publicK);
        int inputLen = data.length;
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        int offSet = 0;
        byte[] cache;
        int i = 0;
        // 对数据分段加密
        while (inputLen - offSet > 0) {
            if (inputLen - offSet > MAX_ENCRYPT_BLOCK) {
                cache = cipher.doFinal(data, offSet, MAX_ENCRYPT_BLOCK);
            } else {
                cache = cipher.doFinal(data, offSet, inputLen - offSet);
            }
            out.write(cache, 0, cache.length);
            i++;
            offSet = i * MAX_ENCRYPT_BLOCK;
        }
        byte[] encryptedData = out.toByteArray();
        out.close();
        return encryptedData;
    }

    public static byte[] encryptByPrivateKey(byte[] data, String privateKey)
            throws Exception {
        byte[] keyBytes = Base64.decodeBase64(privateKey);
        PKCS8EncodedKeySpec pkcs8KeySpec = new PKCS8EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key privateK = keyFactory.generatePrivate(pkcs8KeySpec);
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.ENCRYPT_MODE, privateK);
        int inputLen = data.length;
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        int offSet = 0;
        byte[] cache;
        int i = 0;
        // 对数据分段加密
        while (inputLen - offSet > 0) {
            if (inputLen - offSet > MAX_ENCRYPT_BLOCK) {
                cache = cipher.doFinal(data, offSet, MAX_ENCRYPT_BLOCK);
            } else {
                cache = cipher.doFinal(data, offSet, inputLen - offSet);
            }
            out.write(cache, 0, cache.length);
            i++;
            offSet = i * MAX_ENCRYPT_BLOCK;
        }
        byte[] encryptedData = out.toByteArray();
        out.close();
        return encryptedData;
    }

    public static String getPrivateKey(Map<String, Object> keyMap)
            throws Exception {
        Key key = (Key) keyMap.get(PRIVATE_KEY);
        return Base64.encodeBase64String(key.getEncoded());
    }

    public static String getPublicKey(Map<String, Object> keyMap)
            throws Exception {
        Key key = (Key) keyMap.get(PUBLIC_KEY);
        return Base64.encodeBase64String(key.getEncoded());
    }

    /**
     * 公钥加密
     *
     * @param commodityId
     * @param num
     * @param orderID
     * @param price
     * @param thirdParty
     * @param userAccount
     * @param publicKey
     * @return
     * @throws Exception
     */
    public static String genSign(Long commodityId,
                                 Long num,
                                 String orderID,
                                 Long price,
                                 String thirdParty,
                                 String userAccount,
                                 String publicKey) throws Exception {
        return Base64.encodeBase64String(
                encryptByPublicKey(
                        beforeSign(commodityId, num, orderID, price, thirdParty, userAccount).getBytes("UTF-8"),
                        publicKey
                )
        );
    }

    /**
     * 公钥加密
     *
     * @param orderID
     * @param thirdParty
     * @param userAccount
     * @param publicKey
     * @return
     */
    public static String genSign(String orderID,
                                 String thirdParty,
                                 String userAccount,
                                 String publicKey) throws Exception {
        return Base64.encodeBase64String(
                encryptByPublicKey(
                        beforeSign(orderID, thirdParty, userAccount).getBytes("UTF-8"),
                        publicKey
                )
        );
    }

    /**
     * 签名前字符串
     *
     * @param commodityId
     * @param num
     * @param orderID
     * @param price
     * @param thirdParty
     * @param userAccount
     * @return
     */
    private static String beforeSign(Long commodityId,
                                     Long num,
                                     String orderID,
                                     Long price,
                                     String thirdParty,
                                     String userAccount) {
        return new String("commodityId" + commodityId + "num" + num + "orderID" + orderID + "price" + price + "thirdParty" + thirdParty + "userAccount" + userAccount);
    }

    /**
     * 签名前字符串
     *
     * @param orderID
     * @param thirdParty
     * @param userAccount
     * @return
     */
    public static String beforeSign(String orderID, String thirdParty, String userAccount) {
        return new String("orderID" + orderID + "thirdParty" + thirdParty + "userAccount" + userAccount);
    }

    /**
     * 私钥解密
     *
     * @param sign
     * @param privateKey
     * @return
     */
    private static boolean verifySign(Long commodityId,
                                      Long num,
                                      String orderID,
                                      Long price,
                                      String thirdParty,
                                      String userAccount,
                                      String sign,
                                      String privateKey) {
        try {
            return new String(RSAUtils.decryptByPrivateKey(Base64.decodeBase64(sign), privateKey), "UTF-8").trim().
                    equals(beforeSign(commodityId, num, orderID, price, thirdParty, userAccount));
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 私钥解密
     *
     * @param orderID
     * @param thirdParty
     * @param userAccount
     * @return
     */
    public static boolean verifySign(String orderID,
                                     String thirdParty,
                                     String userAccount,
                                     String sign,
                                     String privateKey) {
        try {
            return new String(RSAUtils.decryptByPrivateKey(Base64.decodeBase64(sign), privateKey), "UTF-8").trim().
                    equals(beforeSign(orderID, thirdParty, userAccount));
        } catch (Exception e) {
            return false;
        }

    }

    public static void main(String[] args) {

        Long commodityId = 1l;
        Long num = 1l;
        Long price = 1l;
        String orderID = "sh1144831826230550";
        String userAccount = "13501134822";
        String thirdParty = "AH_china_mobile";

        //charge order
        try {
            Map<String, Object> keyPair = genKeyPair();
            String privateKey = getPrivateKey(keyPair);     //private key
            String publicKey = getPublicKey(keyPair);       //public key

            String sign = RSAUtils.genSign(commodityId, num, orderID, price, thirdParty, userAccount, publicKey);                                                       //公钥加密
            System.out.println("verify sign result : " + RSAUtils.verifySign(commodityId, num, orderID, price, thirdParty, userAccount, sign, privateKey));             //私钥解密

        } catch (Exception e) {
            e.printStackTrace();
        }

        //charger status
        try {
            Map<String, Object> keyPair = genKeyPair();
            String privateKey = getPrivateKey(keyPair);     //private key
            String publicKey = getPublicKey(keyPair);       //public key

            String sign = RSAUtils.genSign(orderID, thirdParty, userAccount, publicKey);                                                       //公钥加密
            System.out.println("verify sign result : " + RSAUtils.verifySign(orderID, thirdParty, userAccount, sign, privateKey));             //私钥解密

        } catch (Exception e) {
            e.printStackTrace();
        }

    }


}
