package com.chinacreator.video.validate;

import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.CommConfDao;
import com.chinacreator.video.util.IpAddrUtil;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

@Component
public class IpValidate {

    public Logger logger = Logger.getLogger(IpValidate.class);

    @Autowired
    private CommConfDao commConfDao;

    /**
     * IP限制验证
     *
     * @param busitype 业务类型，1查询，0办理
     * @return
     */
    public void validate(String channel, String busitype, String cpid, String spid) throws BussinessException {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        String ip = IpAddrUtil.getIpAddr2(request);
        try {
            //busitype 业务类型，1查询，0办理
            List<Map> list = commConfDao.getIpLimitConf(busitype, channel);

            if (list == null || list.size() == 0) {
                throw new BussinessException("10011", "IP限制无业务配置信息");
            }

            boolean result = false;
            boolean hasExists = false;

            //[{CHANNEL=TX20_wtqd, SPID=1167, IP=172.16.33.16, CPID=youtu}]
            for (Map map : list) {
                //CPID级别验证
                if ("*".equals(map.get("CPID"))) { //不限CPID
                    hasExists = true;
                } else if (cpid != null && cpid.equals(map.get("CPID"))
                        && (spid != null && spid.equals(map.get("SPID")) || "*".equals(map.get("SPID")))) {
                    hasExists = true;
                }

                if (hasExists) {
                    if ("*".equals(map.get("IP"))) {
                        result = true;
                    } else if (map.get("IP") != null && !"".equals(map.get("IP"))) {
                        String[] ips = map.get("IP").toString().split("\\|");
                        for (String str : ips) {
                            if (str != null && ip != null && str.contains(ip)) {
                                result = true;
                            }
                            //如果IP地址配置有*号，访问的IP包含*号前面的部分
                            if (!result && str.indexOf("*") != -1 && ip.startsWith(str.split("[*]")[0])) {
                                result = true;
                            }
                        }
                    }

                    break;
                }
            }
            if (!hasExists) { //无业务配置
                throw new BussinessException("10011", "IP限制无业务配置信息");
            }
            if (!result) {
                throw new BussinessException("10005", "IP验证失败:" + ip);
            }
        } catch (Exception e) {
            e.printStackTrace();
            if (e instanceof BussinessException) {
                throw e;
            } else {
                if (e.getMessage() != null && e.getMessage().indexOf("SQLException") != 0) {
                    throw new BussinessException("10014", "IP限制验证出现SQLException异常。");
                } else {
                    throw new BussinessException("10015", "IP限制验证出现异常，" + e.getMessage());
                }
            }
        }
    }

    public static void main(String[] args) {
        String ip = "222.240.205.10";
        String rule = "111.198.176.234|111.8.57.60|218.76.52.117|111.8.57.60|127.*.*.*|10.199.99.136|10.199.99.141|10.199.99.142|10.199.99.160|10.199.99.161|172.16.123.13|172.16.123.14|172.16.123.15|172.16.123.16|172.16.123.21|172.16.123.22|172.16.123.23|172.16.123.24|222.240.205.10";

        String[] ips = rule.toString().split("\\|");

        boolean result = false;
        for (String str : ips) {
            if (str != null && ip != null && str.contains(ip)) {
                result = true;
                System.out.println(true);
            }
            //如果IP地址配置有*号，访问的IP包含*号前面的部分
            if (!result && str.indexOf("*") != -1 && ip.startsWith(str.split("[*]")[0])) {
                result = true;
            }
        }
    }
}
