package com.chinacreator.video.validate;

import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.CommConfDao;
import com.chinacreator.video.redis.RedisService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;


@Component
public class VisitLimitValidate {

    private final Logger logger = LoggerFactory.getLogger(VisitLimitValidate.class);

    private final static String UNICOM_VIDEO_VISIT_ = "univideo_visit_";

    @Autowired
    private CommConfDao commConfDao;

    @Autowired
    private RedisService redisService;

    public void validate(String channel, String interfaceName) throws Exception {
        try {
            List<Map> visitConf = commConfDao.getlimitQryConf();
            Map visitMap = null;
            if (visitConf != null && visitConf.size() > 0) {
                for (Map map : visitConf) {
                    if (channel.equals(map.get("CHANNEL"))) {
                        visitMap = map;
                        break;
                    }
                }
            }

            if (visitMap != null) {
                String hour = getLimitKey("yyyyMMddHH");
                String day  = getLimitKey("yyyyMMdd");
                String mon  = getLimitKey("yyyyMM");
                String year = getLimitKey("yyyy");

                Object hourLimit = visitMap.get("HOURLIMIT");
                Object dayLimit = visitMap.get("DAYLIMIT");
                Object monthLimit = visitMap.get("MONTHLIMIT");
                Object yearLimit = visitMap.get("YEARLIMIT");
                Object totalLimit = visitMap.get("TOTALLIMIT");

                String prefix = UNICOM_VIDEO_VISIT_ + channel + "_" + interfaceName + "_";

                if (!StringUtils.isEmpty(hourLimit)) {
                    long limitCount = Long.parseLong(hourLimit.toString());

                    Object obj = redisService.hmGet(prefix + day, hour);
                    if (obj != null && Long.parseLong(obj.toString()) >= limitCount) {
                        throw new BussinessException("10031", "访问次数超出最大次数");
                    }
                }

                if (!StringUtils.isEmpty(dayLimit)) {
                    long limitCount = Long.parseLong(dayLimit.toString());

                    Object obj = redisService.hmGet(prefix + mon, day);
                    if (obj != null && Long.parseLong(obj.toString()) >= limitCount) {
                        throw new BussinessException("10032", "访问次数超出最大次数");
                    }
                }

                if (!StringUtils.isEmpty(monthLimit)) {
                    long limitCount = Long.parseLong(monthLimit.toString());

                    Object obj = redisService.hmGet(prefix + year, mon);
                    if (obj != null && Long.parseLong(obj.toString()) >= limitCount) {
                        throw new BussinessException("10033", "访问次数超出最大次数");
                    }
                }

                if (!StringUtils.isEmpty(yearLimit)) {
                    long limitCount = Long.parseLong(yearLimit.toString());

                    Object obj = redisService.get(prefix + "_year_" + year);
                    if (obj != null && Long.parseLong(obj.toString()) >= limitCount) {
                        throw new BussinessException("10034", "访问次数超出最大次数");
                    }
                }

                if (!StringUtils.isEmpty(totalLimit)) {
                    long limitCount = Long.parseLong(totalLimit.toString());

                    Object obj = redisService.get(prefix + "_total_");
                    if (obj != null && Long.parseLong(obj.toString()) >= limitCount) {
                        throw new BussinessException("10035", "访问次数超出最大次数");
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            if (e instanceof BussinessException) {
                throw e;
            } else {
                throw new BussinessException("10030", "验证访问数据限制出现错误，" + e.getMessage());
            }
        }
    }

    /**
     * 设置访问数据
     * [{MONTHLIMIT=6, CHANNEL=TX_2104, HOURLIMIT=2, DAYLIMIT=4, TOTALLIMIT=10, YEARLIMIT=8}]
     * @throws Exception
     */
    public void setVisitcount(String channel, String interfaceName) {
        try {
            Map visitMap = getVisitMap(channel, interfaceName);

            if (visitMap != null) {
                Object hourLimit = visitMap.get("HOURLIMIT");
                Object dayLimit = visitMap.get("DAYLIMIT");
                Object monthLimit = visitMap.get("MONTHLIMIT");
                Object yearLimit = visitMap.get("YEARLIMIT");
                Object totalLimit = visitMap.get("TOTALLIMIT");

                String hour = getLimitKey("yyyyMMddHH");
                String day  = getLimitKey("yyyyMMdd");
                String mon  = getLimitKey("yyyyMM");
                String year = getLimitKey("yyyy");

                String prefix = UNICOM_VIDEO_VISIT_ + channel + "_" + interfaceName + "_";

                if (!StringUtils.isEmpty(hourLimit)) {
                    if (redisService.exists(prefix + day)) {

                        redisService.hincr(prefix + day, hour, 1);

                    } else {

                        redisService.hincr(prefix + day, hour, 1);
                        redisService.expire(prefix + day, 86400 * 2); //设置过期 时间为两天

                    }
                }

                if (!StringUtils.isEmpty(dayLimit)) {
                    if (redisService.exists(prefix + mon)) {

                        redisService.hincr(prefix + mon, day, 1);

                    } else {

                        redisService.hincr(prefix + mon, day, 1);
                        redisService.expire(prefix + mon, 86400 * 62); //设置过期 时间为两个月
                    }
                }

                if (!StringUtils.isEmpty(monthLimit)) {
                    redisService.hincr(prefix + year, mon, 1);
                }

                if (!StringUtils.isEmpty(yearLimit)) {
                    redisService.incr(prefix + "_year_" + year, 1);
                }

                if (!StringUtils.isEmpty(totalLimit)) {
                    redisService.incr(prefix + "_total_", 1);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("设置访问数据出现错误，{}", e.getMessage());
        }
    }

    private Map getVisitMap(String channel, String interfaceName) {
        List<Map> visitConf = commConfDao.getlimitQryConf();
        Map visitMap = null;
        if (visitConf != null && visitConf.size() > 0) {
            for (Map map : visitConf) {
                if (channel.equals(map.get("CHANNEL")) && interfaceName.equals(map.get("INTERFACENAME"))) {
                    visitMap = map;
                    break;
                }
            }
        }

        return visitMap;
    }

    private String getLimitKey(String dateFormat) {
        SimpleDateFormat formatter = new SimpleDateFormat(dateFormat);
        String limitKey = formatter.format(new Date());
        return limitKey;
    }

    public static void main(String[] args) {
        System.out.println(!StringUtils.isEmpty(""));
    }

}
